/****************************************************************************
  Project: NITRO-SampleTools - TXLib - TextureViewer
  File:    main.c

  Copyright 2004,2005 Nintendo Co.,Ltd.	All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  It is a main for TextureViewer.
  Specifications of TextureViewer are summarized below.


	- Display texture easily by following the procedure below.
		1) Copy NTF and textureData.c output by ntexconv in \src\texture.
		2) Copy textureData.h output by ntexconv in \include.
		3) Copy "Add to Makefile" at the end of the header file in Makefile.
		4) Execute make.

	- Display one texture.

	- Switch textures by button operation.

	- Display the operating method by BG.

	- Assign memory as follows:

	Name		Size(KB)		Use application				Address
	VRAM-A		128			Texture texel		0x00000 - 0x20000
	VRAM-B		128			Palette index   	0x20000 - 0x30000
						Texture texel		0x30000 - 0x40000
	VRAM-C		128			None
	VRAM-D		128			BG screen		0x0000 - 0x0800
						BG character		0x0000 - 0x4000
	VRAM-E		64			Texture palette
	VRAM-F		16			OBJ
	VRAM-G		16			None

	- Set 2D as follows:

	Mode
	BG-Mode3

	Name		Use application					scr-size	priority	scr-base	char-base
	BG0			3D										3
	BG1			Menu (16-color text BG)	256x256		0				0			0
	BG2			Information (16-color text BG)	 	256x256		1				1			0
	OBJ			None


	- Maximum texture size that can be displayed is as follows:

	Format name			Maximum graphics size	(Maximum IRIS specification)
	palette4:			1024	* 512			1024	* 1024
	a5i3:				512		* 256			1024	* 512
	palette16:			512		* 512			1024	* 1024
	palette256:			512		* 256			1024	* 512
	direct:				256		* 256			512		* 512
	tex4x4:				512		* 256			1024	* 1024

	Note: The largest possible image size of the tex4x4 here is 1024x512 if the number of colors in the palette is within 32768.

	- Calculate this considering the following data:

	Texel data
	VRAM-A(128KB), slot 0:
	Format name			Maximum graphics size	Used memory (Byte)
	palette4:			1024	* 512		128K
	a5i3:				512		* 256		128K
	palette16:			512 	* 512		128K
	palette256:			512 	* 256		128K
	direct:				256 	* 256		128K
	tex4x4:				1024 	* 512		128K

	Palette index data
	VRAM-B(128KB), slot 1
	Format name			Maximum graphics size	Used memory (Byte)
	tex4x4:				1024 	* 512		64k

	Palette data
	VRAM-E(64KB), slot fixed (0 - 3)
	Format name			Maximum graphics size	Used memory (Byte)
 	palette4:			1024	* 1024		8		(2Byte * 4 colors)
 	a5i3:				1024 	* 512		16		(2Byte * 8 colors)
 	palette16:			1024	* 1024		32		(2Byte * 16 colors)
 	palette256:			1024 	* 512		512		(2Byte * 256 colors)
	tex4x4 (no interpolation):	512 	* 256		64K		(2Byte * 32768 colors)
	tex4x4 (with interpolation):	512 	* 512		64K		(2Byte * 32768 colors)

	Note: The largest image size of the tex4x4 here is the size when the number of colors in the palette is totally uncompressed.

  $Log: main.c,v $
  Revision 1.7  2005/06/20 06:46:11  mominoki
  Updated copyright. Made revision adjustments. 


  Revision 1.2	2004/07/26		mominoki
  Changed .bin to .srl in Makefile's TARGET_BIN.
 Added mainBG.c to DC_FlushRange().

  Revision 1.1	2004/04/15		mominoki
  Fixed comments.
 Changed polygon material color (diffuse).
 Changed sTextureDecalFlg initial setting to DECAL.

  Revision 1.0	2004/01/27		mominoki
  Entered initial version.

  $NoKeywords: $
******************************************************************************/
#include <nitro.h>

#include "myLib/myLib_Os.h"					// NITRO setting
#include "mainBg.h"							// Bg processing
#include "main3D.h"							// 3D processing
#include "textureDataTypeDef.h"
#include "myLib/myLib_fontData.h"

/******************************************************************************
	Global variable
******************************************************************************/
// OS
static u16	sKeyTrigger;					// Key entry (Trigger input)
static u16	sKeyPress;						// Key entry (Hold input)
static u16	sVcounter;						// V-Blank counter

// BG
static u8	sMenuBgRenewFlg;				// Menu BG update flag (0: without, 1: with)
static u8	sMenuBgOnFlg;					// Menu BG display flag (0: without, 1: with)
static u8	sInfoBgOnFlg;					// Information BG display flag (0: without, 1:with)

// Texture
static u8	sTextureDecalFlg;				// Texture DECAL (0:MODULATE, 1:DECAL)
static u8	sPalette0TransparentFlg;		// Texture palette0 transparency flag ( 0:palette0 value, 1: Transparent)
static u8	sTextureLoadFlg;				// Texture load flag ( 0:no, 1: yes)
static u16	sTextureNo;						// Texture number

extern u16 	textureNumber;					// Texture total
extern tTextureData textureArray[];			// Texture data

// Camera
static VecFx32 sEyePos;						// Position
static VecFx32 sEyeAt;						// Eye point
static VecFx32 sEyeUp;						// Upward direction

/******************************************************************************
	Prototype declaration
******************************************************************************/
void	VBlankIntr(void);					// V-Blank interrupt process
void	ReadKey(void);						// Key read

void 	Init(void);							// Initial settings (Initialize global variable)
void 	DrawInfo(void);						// Information display
void	KeyProcess(void);					// Key process

/******************************************************************************
	NitroMain();

	Main
******************************************************************************/
void NitroMain(void)
{
	MyLib_Os_InitNitro();									// Initialize OS
	MyLib_Os_InitVram();									//  Initialize VRAM
	Main3D_InitDisp();										//  Initialize 2D, 3D screen

	MainBg_MenuScr_Init();									// Initialize menu BG screen
	MainBg_InfoScr_Init();									// Initialize information BG screen
	Main3D_Init3DScene();									// Initialize 3D scene

	Init();													// Initialize variables

	MyLib_Os_Intr();										// Set Interrupts, enable

	while (1)
	{
		G3X_Reset();

		ReadKey();											// Read key input
		KeyProcess();										// Key process

		G3_LookAt(&sEyePos, &sEyeUp, &sEyeAt, NULL);
		G3_PushMtx();
		{
			Main3D_RenderPolygon( 							// Draw polygon
				sTextureNo,					// Texture number
				sTextureDecalFlg,			// DECAL flag
				sPalette0TransparentFlg		// palette0 transparency flag
			 );
		}
		G3_PopMtx(1);


		MainBg_InfoScr_ClearAll();							// Clear information BG

		if( sInfoBgOnFlg > 0)
		{
			DrawInfo();										// Information BG ON
		}


		// Memory swap related to geometry and rendering engines
		G3_SwapBuffers(GX_SORTMODE_AUTO, GX_BUFFERMODE_W);

		sVcounter++;
	}
}

/******************************************************************************
	Init();

	Initial settings
******************************************************************************/
void Init(void)
{
	sMenuBgOnFlg = 1;										// Menu BG ON
	sMenuBgRenewFlg = 1;									// Update Menu BG
	sInfoBgOnFlg = 1;										// Information BG ON

	sTextureDecalFlg = 1;									// DECAL
	sPalette0TransparentFlg = 0;
	sTextureNo = 0;											// Texture number
	sTextureLoadFlg =1;										// Load texture

	// Camera
	sEyePos.x 	= 0;
	sEyePos.y 	= 0;
	sEyePos.z 	= 3 * FX32_ONE;

	sEyeAt.x 	= 0;
	sEyeAt.y 	= 0;
	sEyeAt.z 	= 0;

 	sEyeUp.x	= 0;
 	sEyeUp.y	= FX32_ONE;
	sEyeUp.z	= 0;
}

/******************************************************************************
	KeyProcess();

	Key process
******************************************************************************/
void KeyProcess( void )
{
	u16 addPos = FX32_ONE >> 3;

	if( sKeyPress & PAD_KEY_UP )
	{
		if( sKeyPress & PAD_BUTTON_L )
		{
			sEyePos.y	+= addPos;				// Eye point Y axis "+"
			sEyeAt.y	+= addPos;				// Look at point Y axis "+"
		}
		else if(sKeyTrigger & PAD_KEY_UP )
		{
		}
	}
	else if(sKeyPress & PAD_KEY_DOWN )
	{
		if( sKeyPress & PAD_BUTTON_L )
		{
			sEyePos.y	-= addPos;				// Eye point Y axis "-"
			sEyeAt.y	-= addPos;				// Look at point Y axis "-"
		}
		else if(sKeyTrigger & PAD_KEY_DOWN )
		{
		}
	}
	else if( sKeyPress & PAD_KEY_LEFT)
	{
		if( sKeyPress & PAD_BUTTON_L )
		{
			sEyePos.x 	-= addPos;				// Eye point X axis "-"
			sEyeAt.x	-= addPos;				// Look at point X axis "-"
		}
		else if(sKeyTrigger & PAD_KEY_LEFT )
		{

		}
	}
	else if( sKeyPress & PAD_KEY_RIGHT)
	{
		if( sKeyPress & PAD_BUTTON_L )
		{
			sEyePos.x 	+= addPos;				// Eye point X axis "+"
			sEyeAt.x	+= addPos;				// Look at point X axis "+"
		}
		else if(sKeyTrigger & PAD_KEY_RIGHT )
		{

		}
	}
	else if( sKeyPress & PAD_BUTTON_A)
	{
		if( sKeyPress & PAD_BUTTON_L )
		{
			sEyePos.z += addPos;					// Eye point Z axis "+"
		}
		else if(sKeyTrigger & PAD_BUTTON_A )
		{
			if( sTextureNo <(textureNumber - 1))	// Texture number "+"
			{
				sTextureNo++;
				sTextureLoadFlg = 1;
			}
		}
	}
	else if( sKeyPress & PAD_BUTTON_B)
	{
		if( sKeyPress & PAD_BUTTON_L )
		{
			sEyePos.z -= addPos;					// Eye point Z axis "-"
		}
		else if(sKeyTrigger & PAD_BUTTON_B )
		{
			if( sTextureNo >0 )						// Texture number "-"
			{
				sTextureNo--;
				sTextureLoadFlg = 1;
			}
		}
	}
	else if( sKeyTrigger & PAD_BUTTON_START)
	{
		if( sKeyPress & PAD_BUTTON_L )
		{
			sEyePos.x = sEyePos.y = 0;
			sEyePos.z = 3 * FX32_ONE;				// Eye point reset

			sEyeAt.x = sEyeAt.y = sEyeAt.z = 0;		// Look at point reset
		}
		else if(sKeyPress & PAD_BUTTON_R )
		{
			sTextureDecalFlg ^= 1;					// Texture DECAL, MODULATE
		}
		else
		{
			sInfoBgOnFlg ^= 1;						// Information BG ON, OFF
		}
	}
	else if( sKeyTrigger & PAD_BUTTON_SELECT)		// Menu BG ON, OFF
	{
		if(sKeyPress & PAD_BUTTON_R )
		{
			sPalette0TransparentFlg ^= 1;			// palette0 transparency flag
		}
		else
		{
			if( sMenuBgOnFlg > 0 )
			{
				MainBg_MenuScr_ClearAll();				// Clear menu
				sMenuBgOnFlg = 0;						// Switch Menu ON, OFF
			}
			else
			{
				MainBg_MenuScr_Init();					// Initialize BG (read BG dat, set initial BG screen)
				sMenuBgOnFlg = 1;						// Switch Menu ON, OFF
			}
			sMenuBgRenewFlg = 1;						// Update
		}
	}
	else
	{
	}
}

/******************************************************************************
	DrawInfo();

	Display information
******************************************************************************/
void DrawInfo( void )
{
	const tTextureParameter*	pParameter = textureArray[ sTextureNo ].parameter;
	char*	pChar;
	u8		x, y;

	x = 3;
	y = 16;
	MainBg_InfoScr_SetChar(	"Texture No",			x, 				y, 	kFont_Blue );	// Texture number
	MainBg_InfoScr_SetInt(	(u16)(sTextureNo+1),10,	(u8)(x + 12),	y,	kFont_Green);
	MainBg_InfoScr_SetChar(	"/",					(u8)(x + 18),	y, 	kFont_Green);
	MainBg_InfoScr_SetInt( textureNumber, 10,		(u8)(x + 19),	y, 	kFont_Green);

	y++;
	MainBg_InfoScr_SetChar(	"FORMAT",	x, 		y, 		kFont_Blue	);				// Format
	switch( pParameter->type )
	{
		case GX_TEXFMT_NONE:		pChar = "NONE";			break;
		case GX_TEXFMT_PLTT4:		pChar = "palette4";		break;
		case GX_TEXFMT_PLTT16:		pChar = "palette16";	break;
		case GX_TEXFMT_PLTT256:		pChar = "palette256";	break;
		case GX_TEXFMT_A5I3:		pChar = "a5i3";			break;
		case GX_TEXFMT_A3I5:		pChar = "a3i5";			break;
		case GX_TEXFMT_COMP4x4:		pChar = "tex4x4";		break;
		case GX_TEXFMT_DIRECT:		pChar = "direct";		break;
		default: 					pChar = "ERROR";		break;
	}
	MainBg_InfoScr_SetChar(	pChar,	(u8)(x + 12), y, kFont_Green	);

	y++;
	MainBg_InfoScr_SetChar(	"Size S",				x, 				y, kFont_Blue ); 	// Size S
	MainBg_InfoScr_SetInt( pParameter->size_s, 10,	(u8)(x + 12), 	y, kFont_Green );

	y++;
	MainBg_InfoScr_SetChar(	"Size T",				x, 				y, kFont_Blue );		// Size T
	MainBg_InfoScr_SetInt( pParameter->size_t, 10,	(u8)(x + 12), 	y, kFont_Green);

	y++;
	MainBg_InfoScr_SetChar( "Poly Mode", 			x, 				y, kFont_Blue );		// DECAL or MODULATE
	if( sTextureDecalFlg > 0 )
	{
		MainBg_InfoScr_SetChar(	"DECAL", 			(u8)(x + 12), 	y, kFont_Green);
	}else{
		MainBg_InfoScr_SetChar(	"MODULATE",			(u8)(x + 12), 	y, kFont_Green );
	}

	y++;
	MainBg_InfoScr_SetChar( "Palette0", 			x, 				y, kFont_Blue );		// palette0 or transparent
	if( sPalette0TransparentFlg > 0 )
	{
		MainBg_InfoScr_SetChar(	"Not Color", 		(u8)(x + 12), 	y, kFont_Green);
	}
	else
	{
		MainBg_InfoScr_SetChar(	"Color", 			(u8)(x + 12), 	y, kFont_Green);
	}
}

/******************************************************************************
	VBlankIntr();

	Process V-Blank interrupt
******************************************************************************/
void VBlankIntr(void)
{
	if( sMenuBgRenewFlg > 0 )
	{
		MainBg_MenuScr_Renew();								// Update Menu BG
		sMenuBgRenewFlg = 0;
	}

	MainBg_InfoScr_Renew();									// Update information BG

	if( sTextureLoadFlg > 0 )
	{
		Main3D_LoadTexture( sTextureNo );					// Load texture
		sTextureLoadFlg = 0;
	}

	*(u32 *) HW_INTR_CHECK_BUF |= OS_IE_V_BLANK;		   	// Set V-blank interrupt check 
}

/******************************************************************************
	ReadKey();

	Read key
******************************************************************************/
void ReadKey(void)
{
	u16 readData = PAD_Read();

	sKeyTrigger = (u16) (readData & (readData ^ sKeyPress));
	sKeyPress = readData;
}

