/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLib_Data.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  Class that stores 1-byte data.

  $Log: TXLib_Data.cpp,v $
  Revision 1.7  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Corrected comment

  Revision 1.0  2004/03/03		mominoki
  Initial registration

  $NoKeywords: $
 ***************************************************************************/
#define TLB_EXPORT_

#include <cstring>
#include <TXLib/Base/TXLib_Data.h>

namespace std{ using :: memset; }
namespace std{ using :: memcpy; }

namespace TXLib{

/*********************************************************************************************************
	TXLib::CData::CData

	Default constructor
*********************************************************************************************************/
CData::CData()
:
	m_available(false),
	m_byteSize(0),
	m_pData( NULL )
{
}

/*********************************************************************************************************
	TXLib::CData::CData

	Copy constructor
*********************************************************************************************************/
CData::CData(
	const	CData&					rData						// CData of copy source
):
	m_available	(rData.m_available),
	m_byteSize( rData.m_byteSize ),
	m_pData( NULL )
{

	if( m_byteSize > 0 )
	{
		m_pData	= new u8[ rData.m_byteSize ];				// Memory reservation
		std::memcpy( m_pData, rData.m_pData, m_byteSize );	// Data copy
	}
}



/*********************************************************************************************************
	TXLib::CData::operator=

	Assignment Operator
*********************************************************************************************************/
CData&
CData::operator=(
	const	CData&					rData						// CData Pointer to copy source
)
{
	if (&rData != this)
	{
		Clear();
		m_byteSize = rData.m_byteSize;						// Copy byte size

		if (m_byteSize > 0)
		{
			m_available	   = rData.m_available;
			m_pData	= new u8[ rData.m_byteSize ];			// Memory reservation
			std::memcpy( m_pData, rData.m_pData, rData.m_byteSize );	// Copy data
		}
	}
	return *this;
}

/*********************************************************************************************************
	TXLib::CData::~CData

	Destructor
*********************************************************************************************************/
CData::~CData()
{ 
	Clear(); // Clear
}

/*********************************************************************************************************
		public
*********************************************************************************************************/
/*********************************************************************************************************
	TXLib::CData::Clear

	Memory clear
*********************************************************************************************************/
void
CData::Clear()
{
	m_available = false;
	m_byteSize = 0;

	delete[] m_pData;
	m_pData = NULL;
}

/*********************************************************************************************************
	TXLib::CData::Create

	Create

	Create after freeing memory
*********************************************************************************************************/
void
CData::Create(
				u32							byteSize,						// Byte size
				u8							initialData						// Data default value
)
{
	Clear(); // Clear

	m_available = true;
	m_pData = new u8[ byteSize ]; // Memory reservation
	m_byteSize = byteSize;

	std::memset( m_pData, initialData, m_byteSize); // Initialize Memory (Initialize with initialData)
}

/*********************************************************************************************************
	TXLib::CData::GetValue

	Get data
*********************************************************************************************************/
inline void
CData::GetValue( 
				u8*					pData,				// Data pointer
				u32					index				// Number
) const
{
	assert( pData != NULL );
	assert( index < m_byteSize );

	*pData = m_pData[index];  // acquirement
}

/*********************************************************************************************************
	TXLib::CData::SetValue

	Set data
*********************************************************************************************************/
inline void
CData::SetValue( 
				u8					data,				// Data
				u32					index				// Number
) const
{
	assert( index < m_byteSize );

	m_pData[index] = data;  // Set
}

/*********************************************************************************************************
	TXLib::CData::ChangeNumber

	Change byte size

	m_byteSize = byteSize: Does nothing
	m_byteSize > byteSize: Changes byte size after clearing excess.

	m_byteSize < byteSize: Changes the byte size after securing enough memory. Clears the increases to"0".
*********************************************************************************************************/
void
CData::ChangeNumber(
				u32					byteSize				// Byte size of change
)
{
	assert( m_pData != NULL );

	// Too many
	if( m_byteSize > byteSize ){
		std::memset( ( m_pData + byteSize ), 0, ( m_byteSize - byteSize ) ); // Clears excess
		m_byteSize = byteSize; // Change byte size

	// Too few
	}else{
		u8* pData;
		pData= new u8[ byteSize ];
		std::memcpy( pData, m_pData, m_byteSize );							// Copy original
		std::memset( ( pData + m_byteSize), 0, ( byteSize - m_byteSize ) );	// Initialize the rest ot "0"
		
		delete[] m_pData;												// Clear data
		m_pData = NULL;

		m_pData = new u8[ byteSize ];
		m_byteSize = byteSize;
		std::memcpy( m_pData, pData, byteSize );								// Copy

		delete[] pData;													// Clear
		pData = NULL;
	}
}

/*********************************************************************************************************
	TXLib::CData::QuickSortTopDown

	Sort data by quick sort method. 

	If topDown argument is true, line up in descending order. If false, in ascending order.
	It is possible to record how to line up by entering array of index numbers into the pIndex argument  
*********************************************************************************************************/
void
CData::QuickSortTopDown(																				
				u32*								pIndex,						// Index of target data
				u32									firstNumber,				// First Index number of data
				u32									lastNumber,					// Last Index number of data
				bool								topDown						// true: Descending order false: Ascending order
)
{
	// Error assertion
	assert( firstNumber <= lastNumber );

	if( firstNumber == lastNumber ) return;

	// Descending order
	if( topDown )
	{
		for( u32 i=firstNumber; i<lastNumber+1; i++ )
		{
			u32 indexMax;
			u32 dataMax = m_pData[i];
			u32 k = i;

			if( pIndex )
			{
				indexMax = pIndex[i];
			}

			for( u32 j=i+1; j<lastNumber+1; j++ )
			{
				if(m_pData[j] > dataMax )
				{
					if( pIndex )
					{
						indexMax = pIndex[j];
					}
					dataMax = m_pData[j];
					k = j;
				}
			}
			m_pData[k] = m_pData[i];
			m_pData[i] = dataMax;

			if( pIndex )
			{
				pIndex[k] = pIndex[i];
				pIndex[i] = indexMax;
			}
		}
	// Ascending order
	}
	else
	{	
		for( u32 i=firstNumber; i<lastNumber+1; i++ )
		{
			u32 indexMin;
			u32 dataMin = m_pData[i];
			u32 k = i;

			if( pIndex )
			{
				indexMin = pIndex[i];
			}
			for( u32 j=i+1; j<lastNumber+1; j++ )
			{
				if(m_pData[j] < dataMin )
				{
					if( pIndex )
					{
						indexMin = pIndex[j];
					}
					dataMin = m_pData[j];
					k = j;
				}
			}
			m_pData[k] = m_pData[i];
			m_pData[i] = dataMin;

			if( pIndex )
			{
				pIndex[k] = pIndex[i];
				pIndex[i] = indexMin;
			}
		}
	}
}


} // namespace

