/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLib_DefaultColorProcessor_Common.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Log: TXLib_DefaultColorProcessor_Common.cpp,v $
  Revision 1.6  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Corrected comment

  Revision 1.0  2004/03/03		mominoki
  Initial registration

  $NoKeywords: $
 ***************************************************************************/
#define TLB_EXPORT_

#include <cstring>
#include <cassert>
#include <TXLib_Define.h> 
#include <TXLib/Convert/TXLib_DefaultColorProcessor.h>

namespace TXLib{

/*********************************************************************************************************
	TXLib::CDefaultColorProcessor::FindNearPalette

	Searches the palettes for a palette with the same order.

	If found, it returns the beginning of the palette's index. If not found, it returns -1.
*********************************************************************************************************/
int
CDefaultColorProcessor::FindNearPalette(
	const	CColorPalette*			pDstColorPalette,			// Seaching palette
	const	CColorPalette*			pSrcColorPalette,			// SPalettes that are the subject of the search
			u32						nSrcColor,					// Search color number
			u8						commonRed,					// Common color range (Red)
			u8						commonGreen,				// Common color range (Green)
			u8						commonBlue					// Common color range (Blue)
)
{
	assert( (pDstColorPalette != NULL) && (pSrcColorPalette != NULL ) );
	assert( pDstColorPalette->IsAvailable() && pSrcColorPalette->IsAvailable() );
	assert( nSrcColor <= pSrcColorPalette->GetNumber() );

	int	index = -1;

	u8	red, green, blue;									// Color element in search palette
	u8	paletteRed, paletteGreen, paletteBlue;				// Color element in palettes to be searched

	u8	nCompareColor	= pDstColorPalette->GetNumber();	// tex4x4 palette color number

	for( u32 i=0; i<nSrcColor; i+=2)
	{
		u8	existCount		= 0;
		u8	colorRemainder;

		// Searches to see if the same tex4x4 palette exists
		for( u8 n=0; n<nCompareColor; n++ )
		{
			pSrcColorPalette->GetValue( &paletteRed, &paletteGreen, &paletteBlue, i+n );
			pDstColorPalette->GetValue(  &red, &green, &blue , n );			// Get tex4x4 palette color

			colorRemainder = ( red > paletteRed ) ? (red - paletteRed): (paletteRed - red);
			if( colorRemainder > commonRed ) break;			// If even one color is different, another palette

			colorRemainder = ( green > paletteGreen ) ? (green - paletteGreen): (paletteGreen - green);
			if( colorRemainder > commonGreen ) break;		// If even one color is different, another palette

			colorRemainder = ( blue > paletteBlue ) ? (blue - paletteBlue): (paletteBlue - blue);	// Seek color difference
			if( colorRemainder > commonBlue ) break;		// If even one color is different, another palette

			existCount++;
		}

		if( existCount == nCompareColor )
		{
			// Same palette
			index = i;
			break;
		}
	}

	return index;
}

/*********************************************************************************************************
	TXLib::CDefaultColorProcessor::CreateColorMemory

	Create color memory from a direct color image

	If the alpha value exceeds the boundary, add image color to the color palette.
*********************************************************************************************************/
bool
CDefaultColorProcessor::CreateColorMemory(
				CColorPalette*			pColorPalette,
		const	CDirectColorImage*		pDirectColorImage,
		const	CAlphaImage*			pAlphaImage,
				u8						borderValue					// Border value for binarizing alpha values. (This value or above will be opaque)
)
{
	assert( ( pDirectColorImage != NULL ) && ( pColorPalette != NULL ));
	assert( pDirectColorImage->IsAvailable() && pColorPalette->IsAvailable() );

	u32 n=0;
	for( u16 y=0; y<pDirectColorImage->GetHeight(); y++ )
	{
		for( u16 x=0; x<pDirectColorImage->GetWidth(); x++ )
		{
			u8 red, green, blue;

			pDirectColorImage->GetValue( &red, &green, &blue, x, y  );

			if( pAlphaImage )
			{
				u8 alpha;

				pAlphaImage->GetValue( &alpha , x, y );

				if( alpha >= borderValue )
				{
					pColorPalette->SetValue( red, green, blue, n  );
					n++;
				}
			}
			else
			{
				pColorPalette->SetValue( red, green, blue, n );
				n++;
			}
		}
	}
	pColorPalette->ChangeNumber( n );

	return true;
}

/*********************************************************************************************************
	TXLib::CDefaultColorProcessor::CompressPalette_MedhianCut

	Use the median cut method to compress the number of colors.

	Compress the number of colors to the number specified by the user (nNewColor).
*********************************************************************************************************/
void
CDefaultColorProcessor::CompressPalette_MedhianCut( 
			CColorPalette*			pNewColorPalette,			// Newly created palette
	const	CColorPalette* 			pOldColorPalette			// Existing color palette
)
{
	assert( (pOldColorPalette != NULL) && (pNewColorPalette != NULL ) );
	assert( pOldColorPalette->IsAvailable() && pNewColorPalette->IsAvailable() );

	u32 nOldColor	= pOldColorPalette->GetNumber();
	u32 nNewColor	= pNewColorPalette->GetNumber();

	CData	oldRedChannel		( pOldColorPalette->GetRedChannel());
	CData	oldGreenChannel		( pOldColorPalette->GetGreenChannel());
	CData	oldBlueChannel		( pOldColorPalette->GetBlueChannel());

	if( nOldColor <= nNewColor )
	{
		*pNewColorPalette = *pOldColorPalette;
		return;
	}

	// Index memory create
	u32* pBlueIndex		= new u32[ nOldColor ];
	u32* pGreenIndex	= new u32[ nOldColor ];
	u32* pRedIndex		= new u32[ nOldColor ];

	for( u32 i=0; i<nOldColor; i++ )
	{
		pBlueIndex[i] = pGreenIndex[i] = pRedIndex[i] = i;
	}

	u32 indexMax = nOldColor - 1;	// Maximum index number

	// Quick sort the color element array (top down)
	oldRedChannel.QuickSortTopDown(	 pRedIndex,	0, indexMax, false );
	oldGreenChannel.QuickSortTopDown( pGreenIndex,	0, indexMax, false );
	oldBlueChannel.QuickSortTopDown( pBlueIndex,	0, indexMax, false );

	// Seeks the division number (2 to the ? power)
	u32 division = 0;
	u32 divNumber = 0;
	while( division < nNewColor )
	{
		division += (u32)( divNumber * divNumber );
		divNumber++;
	}

	// Repeat the division number
	u32 colorIndex = 0;						// Color palette number

	for( u32 i=0; i<divNumber; i++)
	{
		// The first index is both ends
		u32 index		= 0;
		u32 indexSmall	= 0;
		u32 indexLarge	= indexMax;

		// Until the last index number
		while( indexLarge<nOldColor )
		{
			u8	red, green, blue;
			u8	red2, green2, blue2;
			u8	blueLength,	greenLength, redLength;	// Length of blue, green, and red

			// Seek the length of each color element
			oldRedChannel.GetValue( &red,	indexLarge );
			oldRedChannel.GetValue( &red2,	indexSmall );
			redLength = red - red2;

			oldGreenChannel.GetValue( &green,	indexLarge );
			oldGreenChannel.GetValue( &green2,	indexSmall );
			greenLength = green - green2;

			oldBlueChannel.GetValue( &blue,		indexLarge );
			oldBlueChannel.GetValue( &blue2,	indexSmall );
			blueLength = blue - blue2;

			// From the longest element, set the distribution center to be the index.
			u32 middleIndex = ( indexSmall + indexLarge ) / 2;			// sample value center
			index			= pBlueIndex[ middleIndex ];				// Initial is blue

			if( ( greenLength > blueLength ) && ( greenLength > redLength ))
			{
				index = pGreenIndex[ middleIndex ];
			}
			else if( ( redLength > blueLength ) && ( redLength > greenLength ) )
			{
				index = pRedIndex[ middleIndex ];
			}

			pOldColorPalette->GetValue( &red,	&green, &blue, index );
			pNewColorPalette->SetValue( red,	green,	blue, colorIndex  );

			colorIndex++;

			if( colorIndex >= nNewColor )	// Quit when it becomes the number of colors sought.
			{
				break;
			}

			indexSmall = indexLarge;
			indexLarge += indexMax;
		}

		if( colorIndex >= nNewColor )	// Quit when it becomes the number of colors sought.
		{
			break;
		}

		indexMax /= 2;	// Index maximum value change
	}

	delete[] pBlueIndex;
	delete[] pGreenIndex;
	delete[] pRedIndex;
}

} //namespace