/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLib_Ntf.h

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This class inputs NTF(Iris Texture File) data into CTexture, or outputs CTexture data to NTF data.
  CNtf derives CTexture.

  $Log: TXLib_Ntf.cpp,v $
  Revision 1.7  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Corrected comment

  Revision 1.0  2004/03/03		mominoki
  Initial registration

  $NoKeywords: $
 ***************************************************************************/
#define TLB_EXPORT_

#include <cstdio>
#include <cstring>
#include <TXLib_Define.h> 
#include <TXLib/Base/TXLib_Data.h>
#include <TXLib/Base/TXLib_ByteData.h>
#include <TXLib/Base/TXLib_Texture.h>
#include <TXLib/Convert/TXLib_Ntf.h>

namespace std{ using :: memset; }

namespace TXLib{


/*********************************************************************************************************
		public
*********************************************************************************************************/
/*********************************************************************************************************
	TXLib::CNtf::GetNearNTFSize

	Get NTF size that is near the current size.

	* If succeed, return one of the following: 8, 16, 32, 64, 128, 256, 512, 1024texel.
	* If error, (0 or greater than 1024), return -1.
*********************************************************************************************************/
s32
CNtf::GetNearNTFSize(
		 u16			size					// Source texture
) const
{
	if( ( size == 0 ) || ( size > kDefine_sizeMax ) ) return -1;

	u16	remainder;								// Difference
	u16	remainderMin	= 0xffff;				// Difference minimum value
	s32 nearSize		= 8;					// Near value
	u16	i				= kDefine_sizeMax;		// Width. Maximum value

	while( i > 7 )		
	{
		remainder = i - size;

		if( remainder < remainderMin )
		{
			remainderMin = remainder;
			nearSize = i;
			if( remainder == 0 ) break;
		}

		i >>= 1;
	}

	return nearSize;
}

/*********************************************************************************************************
	TXLib::CNtf::Write()

	Write NTF data to memory.
*********************************************************************************************************/
bool
CNtf::Write( 
			tMemory*					pNTFMemory,						// tMemory
			CTexture*					pTexture,						// CTexture pointer
			bool						sizeAlignmentFlg				// NTF size alignment flag
)const
{
	assert(( pNTFMemory != NULL ) && (pTexture != NULL ));

	CTexture::tType	type	= pTexture->GetType();		// Texture format
	u16				width	= pTexture->GetWidth();
	u16				height	= pTexture->GetHeight();

	// Align texture size to NTF size
	if( sizeAlignmentFlg )
	{
		s32 size;

		if( ( size = GetNearNTFSize( pTexture->GetWidth() ) ) == -1 ) return false;
		width = (u16)size;

		if( ( size = GetNearNTFSize( pTexture->GetHeight() ) ) == -1 ) return false;
		height = (u16)size;

		if( !pTexture->AlignmentImageSize( width, height ) ) return false;
	}

	// Memory clear
	std::memset(pNTFMemory->pTexelData,			0, pNTFMemory->texelDataByteSize);
	std::memset(pNTFMemory->pPaletteData,		0, pNTFMemory->paletteDataByteSize);
	std::memset(pNTFMemory->pPaletteIndexData,	0, pNTFMemory->paletteIndexDataByteSize);

	u8	bitCount;		// Bit depth
	u8	index;
	u16 data_16bit;
	u32 x, y;
	u32 offset;
	u32 number;
	u8 red, green, blue, alpha;

	if( pTexture->IsPaletteType() )
	{
		// Determine bit depth to convert
		if( type == CTexture::kType_palette4 )
		{
			bitCount		= 2;
		}
		else if( ( type == CTexture::kType_a5i3 ) || ( type == CTexture::kType_a3i5 ) || ( type == CTexture::kType_palette256 ) )
		{
			bitCount		= 8;
		}
		else if(  type == CTexture::kType_palette16  )
		{
			bitCount		= 4;
		}
		else
		{
			return false;
		}

		// Color index
		if( pNTFMemory->pTexelData == NULL ) return false;

		for(y=0; y<height; y++)
		{
			for(x=0; x<width; x++)
			{
				pTexture->GetColorIndexImage().GetValue( &index, x, y ); // Get color index value

				// Add alpha to texel data
				if( type == CTexture::kType_a5i3 )
				{
					pTexture->GetAlphaImage().GetValue( &alpha, x, y ); // Get alpha value
					alpha >>= 3; // Alpha 8bit -> 5bit convert
					index |= (u32)( alpha << 3 ); // Add alpha value to color index value
				}
				else if( type == CTexture::kType_a3i5 )
				{
					pTexture->GetAlphaImage().GetValue( &alpha, x, y ); // Get alpha value
					alpha >>= 5; // Alpha 8bit -> 3bit convert
					index |= (u32)( alpha << 5 ); // Added alpha value to color index value
				}
				switch( bitCount )
				{
					case 1:
						offset = ( ( y * width / 16 ) + x / 16 ) * 2;
						data_16bit = ( (u16)index << ( x % 16 ) );
						*(u16*)( pNTFMemory->pTexelData + offset ) |= data_16bit;
						break;
					case 2:
						offset = ( ( y * width / 8 ) + x / 8 ) * 2;
						data_16bit = ( (u16)index << ( ( x % 8 ) * 2 ) );
						*(u16*)( pNTFMemory->pTexelData + offset ) |= data_16bit;
						break;
					case 4:
						offset = ( ( y * width / 4 ) + x / 4 ) * 2;
						data_16bit = ( (u16)index << ( ( x % 4 ) * 4 ) );
						*(u16*)( pNTFMemory->pTexelData + offset ) |= data_16bit;
						break;
					case 8:
						offset = ( ( y * width / 2 ) + x / 2 ) * 2;
						data_16bit = ( (u16)index << ( ( x % 2 ) * 8 ) );
						*(u16*)( pNTFMemory->pTexelData + offset ) |= data_16bit;
						break;
					default:
						return false;
				}
			}
		}

		// Palette
		if( pNTFMemory->pPaletteData == NULL ) return false;
		u32 nColor = pTexture->GetColorPalette().GetNumber();
//		pNTFMemory->paletteDataByteSize = nColor * 2;	// Match byte size to the number of palettes

		for( number=0; number<nColor; number++ )
		{
			pTexture->GetColorPalette().GetValue( &red, &green, &blue, number ); // Get color value

			// 8bit->5bit conversion
			red >>= 3;
			green >>= 3;
			blue >>= 3;

			data_16bit =  TLB_DATA_1555( 0, blue, green, red); // Reorder R,G,B order. Create palette color data.

			offset = number * 2;
			*(u16*)( pNTFMemory->pPaletteData + offset ) = data_16bit; // Set to memory
		}
	}
	else if( type == CTexture::kType_tex4x4 )
	{
		u32 dataOffset; // Data offset inside block (2 - 30-bit)
		u32 blockOffset; // Off set by block
		u8 colorIndex; // Color index
		u32 data_32bit;
		u32 index;
		u32 texelData; // Texel data

		// Texel data
		for( u32 y=0; y<height; y += TLB_4 )
		{
			for( u32 x=0, block=0; x<width; x += TLB_4, block++ )
			{
				index = 0;
				texelData = 0;
				blockOffset = 0;
				for( u32 block_y=0; block_y<TLB_4; block_y++)
				{
					for( u32 block_x=0; block_x<TLB_4; block_x++)
					{
						pTexture->GetColorIndexImage().GetValue( &colorIndex, (x + block_x), (y + block_y) ); // Get color index

						blockOffset = ( ( width / 4 ) * ( y / 4 ) + ( x / 4 ) ) * 4;
						dataOffset = ( ( block_y % 4 ) * 4 + ( block_x % 4 ) ) * 2;
						data_32bit = colorIndex << dataOffset;
						texelData |=  data_32bit;
					}
				}
				*(u32*)( pNTFMemory->pTexelData +  blockOffset ) = texelData;
			}
		}

		// Palette index
		u32 paletteAddress;
		u8	attributeData; // Palette attribute
		u32	number = pTexture->GetColorPaletteAttribute().GetByteSize();

		for( u32 i=0; i<number; i++)
		{
			pTexture->GetColorPaletteAttribute().GetValue( &attributeData, i  ); // Get palette attribute
			pTexture->GetColorPaletteAddress().GetValue( &paletteAddress, i ); // Get palette address

			// Palette index
			*(u16*)( pNTFMemory->pPaletteIndexData + ( i * 2 ) ) = ( (u8)attributeData << 14 ) | (u16)paletteAddress;
		}

		// Palette
		u32 nColor = pTexture->GetColorPalette().GetNumber();
		for( u32 number=0; number<nColor; number++ )
		{
			pTexture->GetColorPalette().GetValue( &red, &green, &blue, number ); // Get color value

			// 8bit->5bit conversion
			red >>= 3;
			green >>= 3;
			blue >>= 3;

			data_16bit =  TLB_DATA_1555( 0, blue, green, red); // Reorder R,G,B order. Create palette color data.

			offset = number * 2;
			*(u16*)( pNTFMemory->pPaletteData + offset ) = data_16bit; // Set to memory
		}
	}
	else if( type == CTexture::kType_direct )
	{
		if( pNTFMemory->pTexelData == NULL ) return false;

		number = 0;
		for(y=0; y<height; y++)
		{
			for(x=0; x<width; x++, number++)
			{
				pTexture->GetDirectColorImage().GetValue( &red, &green, &blue, x, y  ); // Get color value of image
				pTexture->GetAlphaImage().GetValue( &alpha, x, y ); // Get alpha value

				// 8bit->5bit conversion
				red >>= 3;
				green >>= 3;
				blue >>= 3;
				alpha = (alpha > 0 ) ? 1: 0; // Alpha 8bit -> 1bit conversion (if alpha exists, "1")
				data_16bit =  TLB_DATA_1555( alpha, blue, green, red); // Reorder R,G,B order. Create palette color data.
		
				offset = number * 2;
				*(u16*)( pNTFMemory->pTexelData + offset ) = data_16bit; // Set to memory
			}
		}

	}
	else
	{
		return false;
	}

	return true;
}


/*********************************************************************************************************
	TXLib::CNtf::GetDataByteSize()

	Get memory size required for Ntf memory output
*********************************************************************************************************/
bool
CNtf::GetDataByteSize(
					u32*						pTexelDataByteSize,				// Texel data byte size
					u32*						pPaletteDataByteSize,			// Palette data byte size
					u32*						pPaletteIndexDataByteSize,		// Palette index byte size
					CTexture*					pTexture,						// CTexture pointer
					bool						sizeAlignmentFlg				// NTF size alignment flag
) const
{
	assert( (pTexelDataByteSize != NULL) && (pPaletteDataByteSize != NULL) && (pPaletteIndexDataByteSize != NULL) &&
			( pTexture != NULL ));

	CTexture::tType		type	= pTexture->GetType();		// Texture format
	u16					width	= pTexture->GetWidth();		// Width
	u16					height	= pTexture->GetHeight();	// Height

	u8 texelBitCount		= 0; // Texel data bit depth
	u32 nPaletteColorIndex	= 0; // Number of palette index
	u32 nPaletteColor	= (pTexture->GetColorPalette().IsAvailable()) ? pTexture->GetColorPalette().GetNumber(): 0; // Number of colors in the palette

	// Align texture size to NTF size
	if( sizeAlignmentFlg )
	{
		s32 size;

		if( ( size = GetNearNTFSize( pTexture->GetWidth() ) ) == -1 ) return false;
		width = (u16)size;

		if( ( size = GetNearNTFSize( pTexture->GetHeight() ) ) == -1 ) return false;
		height = (u16)size;
	}

	switch( type )
	{
		case CTexture::kType_palette4:
			texelBitCount		= 2;	break;
		case CTexture::kType_palette16:
			texelBitCount		= 4;	break;
		case CTexture::kType_a5i3:
		case CTexture::kType_a3i5:
		case CTexture::kType_palette256:
			texelBitCount		= 8;	break;
		case CTexture::kType_tex4x4:
			texelBitCount		= 2;
			nPaletteColorIndex	= ( width * height ) / ( TLB_4 * TLB_4);
			break;
		case CTexture::kType_direct:
			texelBitCount		= 16;	break;
		default:
			false;
	}

	*pTexelDataByteSize			= ( width * height *  texelBitCount ) / 8;	// Number of texels * bit depth / 8 bits
	*pPaletteDataByteSize		= nPaletteColor * sizeof(u16);				// Number of colors in the palette * 2 Bytes (RGB555)
	*pPaletteIndexDataByteSize	= nPaletteColorIndex * sizeof(u16);			// Number of palette indexes * 2 Bytes.

	return true;
}

} // namespace
