/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLibUtility_Bmp.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This is a class for inputting BMP data to CTexture, and outputting CTexture data as BMP data.

  $Log: TXLibUtility_Bmp.cpp,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Revised copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Fixed comments.

  Revision 1.0  2004/03/03		mominoki
  Entered initial version.

  $NoKeywords: $
 ***************************************************************************/
#include "stdafx.h"
#include <cstring>
#include "../include/TXLibUtility_Bmp.h"

namespace std{ using :: memset; }
namespace std{ using :: memcpy; }

namespace TXLibUtility{

/*********************************************************************************************************
		private
*********************************************************************************************************/
/********************************************************************************************************
	TXLibUtility::CBmp::GetNeedLine()

	Get nexessary number of lines.
	Convert the lines into DWORD alignment.
*********************************************************************************************************/
u16
CBmp::GetNeedLine(
			u16						bitCount,					// Bit depth
			u16						width						// Width
)const
{
	u32 bitLine = width * bitCount;
	u32 byteLine = ( bitLine % 8 ) ? ( bitLine / 8 + 1 ) : ( bitLine / 8 );
	u32 needLine = byteLine;

	if( needLine % 4 )
	{
		needLine = ( byteLine / 4 + 1 ) * 4;
	}
	return needLine;
}

/********************************************************************************************************
	TXLibUtility::CBmp::ReadBMPHEADER()

	Load BMP header

	The conditions for BMP loading are as follows:

	- Must not be RLE compressed 

	Return value:  bool
*********************************************************************************************************/
bool
CBmp::ReadBMPHEADER( 
			tFileHeader*	pFileHeader,					// BITMAPFILEHEADER pointer
			tInfoHeader*	pInfoHeader,					// BITMAPINFOHEADER pointer
	const	u8*				pMemory							// Input data pointer
)
{
	assert( ( pMemory != NULL ) && ( pFileHeader != NULL ) && ( pInfoHeader != NULL ) );

	u32 dataOffset = 0;

	// FILEHEADER
	pFileHeader->bfType			= *(u16*)( pMemory + dataOffset );	// Format
	dataOffset += 2;

	pFileHeader->bfSize			= *(u32*)( pMemory + dataOffset );	// File size
	dataOffset += 4;

	pFileHeader->bfReserved1	= *(u16*)( pMemory + dataOffset );	// Reserve
	dataOffset += 2;

	pFileHeader->bfReserved2	= *(u16*)( pMemory + dataOffset );	// Reserve
	dataOffset += 2;

	pFileHeader->bfOffBits		= *(u32*)( pMemory + dataOffset );	// Image offset
	dataOffset += 4;

	// INFOHEADER
	pInfoHeader->biSize			= *(u32*)( pMemory + dataOffset );	// INFOHEADER Size
	dataOffset += 4;

	pInfoHeader->biWidth		= *(u32*)( pMemory + dataOffset );	// Width
	dataOffset += 4;

	pInfoHeader->biHeight		= *(u32*)( pMemory + dataOffset );	// Height
	dataOffset += 4;

	pInfoHeader->biPlanes		= *(u16*)( pMemory + dataOffset );	// Plane
	dataOffset += 2;

	pInfoHeader->biBitCount		= *(u16*)( pMemory + dataOffset );	// Bit depth
	dataOffset += 2;

	pInfoHeader->biCompression	= *(u32*)( pMemory + dataOffset );	// Compression
	dataOffset += 4;

	pInfoHeader->biSizeImage	= *(u32*)( pMemory + dataOffset );	// Image size
	dataOffset += 4;

	pInfoHeader->biXPelsPerMeter = *(u32*)( pMemory + dataOffset );	// X resolution
	dataOffset += 4;

	pInfoHeader->biYPelsPerMeter = *(u32*)( pMemory + dataOffset );	// Y resolution
	dataOffset += 4;

	pInfoHeader->biClrUsed		= *(u32*)( pMemory + dataOffset );	// Used color palette number
	dataOffset += 4;

	pInfoHeader->biClrImportant = *(u32*)( pMemory + dataOffset );	// Important color palette number
	dataOffset += 4;

	if( pFileHeader->bfType != 0x4d42 ) return false;				// "BM" character acknowledgement
	if( pInfoHeader->biCompression != 0 ) return false;				// Compression format

	return true;
}


/********************************************************************************************************
	TXLibUtility::CBmp::WriteBMPHEADER()

	Write BMP header
*********************************************************************************************************/
bool
CBmp::WriteBMPHEADER(
			u8*						pMemory,					// Output data pointer
			u32*					pDataSize,					// Output data size pointer
			u16						bitCount,					// Bit depth
			u16						width,						// Width
			u16						height,						// Height			
			u32						nColorUsed					// Actual number of palette colors
) const
{
	assert( pMemory != NULL );

	tFileHeader	fileHeader;					// BITMAPFILEHEADER pointer
	tInfoHeader	infoHeader;					// BITMAPINFOHEADER pointer

	infoHeader.biSizeImage = GetNeedLine( bitCount, width ) * height;	// Image size

	switch( bitCount )
	{
		case 1:
			infoHeader.biClrUsed		= 0;								// Number of colors used in color palette
			infoHeader.biClrImportant	= 0;								// Number of palette colors
			fileHeader.bfOffBits		= kDefine_fileHeaderByteSize + kDefine_infoHeaderByteSize + 
											(2 * 4);		// Image offset
			break;
		case 4:
			infoHeader.biClrUsed		= nColorUsed;								// Number of colors used in color palette
			infoHeader.biClrImportant	= ( nColorUsed == 0 ) ? 16: nColorUsed;		// Number of colors in color palette
			fileHeader.bfOffBits		= kDefine_fileHeaderByteSize + kDefine_infoHeaderByteSize + 
											(infoHeader.biClrImportant * 4);		// Image offset
			break;
		case 8:
			infoHeader.biClrUsed		= nColorUsed;								// Number of colors used in color palette
			infoHeader.biClrImportant	= ( nColorUsed == 0 ) ? 256: nColorUsed;	// Number of colors in color palette
			fileHeader.bfOffBits		= kDefine_fileHeaderByteSize + kDefine_infoHeaderByteSize + 
											(infoHeader.biClrImportant * 4);		// Image offset
			break;
		case 24:
			infoHeader.biClrImportant	= infoHeader.biClrUsed = 0;					// Number of palette colors
			fileHeader.bfOffBits		= kDefine_fileHeaderByteSize + kDefine_infoHeaderByteSize; // Image offset
			break;
		default:
			return false;
	}

	// tFileHeader
	fileHeader.bfType			= 0x4d42;	// 'BM'
	fileHeader.bfReserved1		= 0;		// Reserve 1
	fileHeader.bfReserved2		= 0;		// Reserve 2
	fileHeader.bfSize			= fileHeader.bfOffBits + infoHeader.biSizeImage; // File byte size = Image offset + image size
	
	if( pDataSize )
	{
		*pDataSize = fileHeader.bfSize;
	}

	// tInfoHeader
	infoHeader.biBitCount		= bitCount;						// Bit depth
	infoHeader.biSize			= kDefine_infoHeaderByteSize;	// tInfoHeader size
	infoHeader.biWidth			= width;						// Width
	infoHeader.biHeight			= height;						// Height
	infoHeader.biPlanes			= 1;							// Fixed
	infoHeader.biCompression	= 0;							// No RLE compression

// Need to accurately determine resolution
	infoHeader.biXPelsPerMeter	= 0;		// Resolution X
	infoHeader.biYPelsPerMeter	= 0;		// Resolution Y


	u32 dataOffset = 0;
	
	// FILEHEADER Write
	*(u16*)( pMemory + dataOffset )	= fileHeader.bfType;
	dataOffset += 2;

	*(u32*)( pMemory + dataOffset ) = fileHeader.bfSize;
	dataOffset += 4;

	*(u16*)( pMemory + dataOffset ) = fileHeader.bfReserved1;
	dataOffset += 2;

	*(u16*)( pMemory + dataOffset ) = fileHeader.bfReserved2;
	dataOffset += 2;

	*(u32*)( pMemory + dataOffset ) = fileHeader.bfOffBits;
	dataOffset += 4;

	// INFOHEADER Write
	*(u32*)( pMemory + dataOffset ) = infoHeader.biSize;
	dataOffset += 4;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biWidth;
	dataOffset += 4;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biHeight;
	dataOffset += 4;

	*(u16*)( pMemory + dataOffset ) = infoHeader.biPlanes;
	dataOffset += 2;

	*(u16*)( pMemory + dataOffset ) = infoHeader.biBitCount;
	dataOffset += 2;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biCompression;
	dataOffset += 4;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biSizeImage;
	dataOffset += 4;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biXPelsPerMeter;
	dataOffset += 4;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biYPelsPerMeter;
	dataOffset += 4;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biClrUsed;
	dataOffset += 4;

	*(u32*)( pMemory + dataOffset ) = infoHeader.biClrImportant;
	dataOffset += 4;

	return true;
}

/********************************************************************************************************
	TXLibUtility::CBmp::ReadDirectColorImage()

	Input memory data to DirectColorImage

		The BMP data line goes from bottom to top, so set Y so that it starts at the top left.

		BMP Data		CTexture Format
		direct -> TXLib::CTexture::kType_direct
*********************************************************************************************************/
void
CBmp::ReadDirectColorImage(
			TXLib::CDirectColorImage*	pDirectColorImage,			// Direct color image pointer
	const	u8*							pMemory,					// Input data pointer
			u16							width,						// Width
			u16							height						// Height		
)
{
	assert(( pDirectColorImage != NULL ) && ( pMemory != NULL ));

	pDirectColorImage->Create( width, height, 0, 0, 0 );

	u16 needLine	= GetNeedLine( 24, width );
	u16 dummyLine	= needLine - ( width * 3 );

	for( u16 y=0; y<height; y++)
	{
		for( u16 x=0; x<width; x++ )
		{
			u32 dataOffset = (height - 1 - y) * (width * 3 + dummyLine) + (x * 3);

			u8 blue	=	*( pMemory + dataOffset     );				// Get
			u8 green=	*( pMemory + dataOffset + 1 );
			u8 red =	*( pMemory + dataOffset + 2 );
			
			pDirectColorImage->SetValue( red, green, blue, x, y );	// Set
		}
	}
}

/********************************************************************************************************
	TXLibUtility::CBmp::WriteDirectColorImage()

	Output DirectColorImage to memory

		Arrange BMP from bottom to top and input to memory.

			CBmp Format						BMP Data
		TXLib::CTexture::kType_direct	 -> direct
*********************************************************************************************************/
void
CBmp::WriteDirectColorImage(
			u8*							pMemory,					// Output data pointer
	const	TXLib::CDirectColorImage*	pDirectColorImage			// Direct color image pointer
) const
{
	assert(  ( pMemory != NULL ) && ( pDirectColorImage != NULL ));
	assert(	pDirectColorImage->IsAvailable());

	u16 width	= pDirectColorImage->GetWidth();
	u16 height	= pDirectColorImage->GetHeight();
	u16 needLine	= GetNeedLine( 24, width );
	u16 dummyLine	= needLine - ( width * 3 );
	u8 red, green, blue;

	for( u16 y=0; y<height; y++)
	{
		for( u16 x=0; x<width; x++ )
		{
			pDirectColorImage->GetValue( &red, &green, &blue, x, y );	// Get

			u32 dataOffset = (height - 1 - y) * (width * 3 + dummyLine) + (x * 3);

			*( pMemory + dataOffset     ) = blue;						// Set
			*( pMemory + dataOffset + 1 ) = green;
			*( pMemory + dataOffset + 2 ) = red;				
		}
	}
}

/********************************************************************************************************
	TXLibUtility::CBmp::ReadColorPalette()

	Input memory data to ColorPalette
*********************************************************************************************************/
void
CBmp::ReadColorPalette(
			TXLib::CColorPalette*	pColorPalette,					// Color Palette Pointer
	const	u8*						pMemory,						// Input Data Pointer
			u32						nColorUsed						// Actual number of pelette colors
) 
{
	assert((pColorPalette != NULL ) && ( pMemory != NULL ));

	pColorPalette->Create( nColorUsed, 0, 0, 0 );

	for( u32 i=0; i<nColorUsed; i++)
	{
		u8 blue =	*( pMemory + ( i * 4 )     );			// Get
		u8 green =	*( pMemory + ( i * 4 ) + 1 );
		u8 red =	*( pMemory + ( i * 4 ) + 2 );

		pColorPalette->SetValue( red, green, blue, i  );	// Set
	}
}

/********************************************************************************************************
	TXLibUtility::CBmp::WriteColorPalette()

	Output ColorPalette to memory
*********************************************************************************************************/
void
CBmp::WriteColorPalette(
			u8*						pMemory,						// Output data pointer
	const	TXLib::CColorPalette*	pColorPalette					// Color palette pointer
) const
{
	assert(( pMemory != NULL ) && ( pColorPalette != NULL ));
	assert( pColorPalette->IsAvailable());

	for( u32 i=0; i<pColorPalette->GetNumber(); i++)
	{
		u8 red, green, blue;

		pColorPalette->GetValue( &red, &green, &blue, i  );	// Get

		*( pMemory + ( i * 4 )) = blue;						// Set
		*( pMemory + ( i * 4 ) + 1 ) = green;
		*( pMemory + ( i * 4 ) + 2 ) = red;
		*( pMemory + ( i * 4 ) + 3 ) = 0; // reserve
	}
}

/********************************************************************************************************
	TXLibUtility::CBmp::ReadColorIndexImage()

	Input memory data to ColorIndexImage

		The BMP data line goes from bottom to top, so set Y so that it starts at the top left.
*********************************************************************************************************/
bool
CBmp::ReadColorIndexImage(
			TXLib::CColorIndexImage*	pColorIndexImage,			// Color index image
	const	u8*							pMemory,					// Input data pointer
			u16							bitCount,					// Bit depth
			u16							width,						// Width
			u16							height						// Height
) 
{
	assert(( pColorIndexImage != NULL ) && ( pMemory != NULL ));

	pColorIndexImage->Create( width, height);

	u8 data;
	u32 dataOffset;

	u16 needLine	= GetNeedLine( bitCount, width );
	u16 dummyLine	= needLine - ( width / ( 8 / bitCount ) );

	for( u16 y=0; y<height; y++)
	{
		for( u16 x=0; x<width; x++ )
		{
			switch( bitCount ) //Bit width conversion ( 1texel xbit -> 8bit )
			{
				case 1:
					dataOffset = (height - 1 - y) * ( width / 8 + dummyLine ) + x / 8; // Data offset computation
					data	= *( pMemory + dataOffset ); // Get
					data >>= ( 7 - x % 8 );
					data &= 0x01;
					break;
				case 4:
					dataOffset	= (height - 1 - y) * ( width / 2 + dummyLine) + x / 2; // Data offset computation
					data	= *( pMemory + dataOffset ); // Get
					data >>= ( 1 - x % 2) * 4;
					data &= 0x0f;
					break;
				case 8:
					dataOffset	= (height - 1 - y) * (width + dummyLine) + x; // Data offset computation
					data		= *( pMemory + dataOffset ); // Get
					break;
				default:
					return false;
			}	
			pColorIndexImage->SetValue( data, x, y );	// Set
		}
	}

	return true;
}

/********************************************************************************************************
	TXLibUtility::CBmp::WriteColorIndexImage()

	Output ColorIndexImage to memory

	The BMP data line goes from bottom to top, so set Y so that it starts at the top left.
*********************************************************************************************************/
void
CBmp::WriteColorIndexImage(
			u8*							pMemory,						// Output data pointer
	const	TXLib::CColorIndexImage*	pColorIndexImage,				// Color index image
			u16							bitCount						// Bit depth
) const
{
	assert( ( pMemory != NULL ) &&  ( pColorIndexImage != NULL ));
	assert(	pColorIndexImage->IsAvailable());

	u16 width	= pColorIndexImage->GetWidth();
	u16 height	= pColorIndexImage->GetHeight();

	// Byte size computation
	u32 byteSize = GetNeedLine( bitCount, width ) * height;

	// Reserve and initialize memory buffer
	u8* pMemoryBuf;
	pMemoryBuf = new u8[ byteSize ];
	std::memset( pMemoryBuf, 0, byteSize );

	u32 dataOffset;
	u8 data, index;
	u16 needLine	= GetNeedLine( bitCount, width );
	u16 dummyLine	= needLine - ( width / ( 8 / bitCount ) );

	for( u16 y=0; y<height; y++)
	{
		for( u16 x=0; x<width; x++ )
		{
			pColorIndexImage->GetValue( &index, x, y );	// Get

			switch( bitCount ) //Bit width conversion ( 1texel xbit -> 8bit )
			{
				case 1:
					dataOffset = (height - 1 - y) * ( width / 8 + dummyLine ) + x / 8; // Data offset computation
					data	= *( pMemoryBuf + dataOffset ); // Get
					data |= ( index << ( 7 - x % 8 ));
					break;
				case 4:
					dataOffset	= (height - 1 - y) * ( width / 2 + dummyLine) + x / 2; // Data offset computation
					data	= *( pMemoryBuf + dataOffset ); // Get
					data |= ( index << ( ( 1 - x % 2 ) * 4 ) );
					break;
				case 8:
					dataOffset	= (height - 1 - y) * (width + dummyLine) + x; // Data offset computation
					data = index;
					break;
				default:
					assert( false );
			}	
			*( pMemoryBuf + dataOffset ) = data; // Set
		}
	}

	// Memory copy
	std::memcpy( pMemory, pMemoryBuf, byteSize );

	delete[] pMemoryBuf; // Clear memory
	pMemoryBuf = NULL;
}

/*********************************************************************************************************
		public
*********************************************************************************************************/
/********************************************************************************************************
	TXLibUtility::CBmp::Read()

	Load BMP data and create CBMP.


		Computes and substitutes when the biClrUsed (the actual number of palette colors) of tInfoHeader is 0.
		The computation method is the byte size of the palette / 4. (4 bytes per color) 

		Computes and substitutes when the biSizeImage (the byte size of the image) of tInfoHeader is 0.
		The computation method is the total file size - the offset to the image 

		 palette2	->	TXLib::CTexture::kType_palette4
		 palette16	->	TXLib::CTexture::kType_palette16
		 palette256 ->	TXLib::CTexture::kType_palette256
		 direct		->	TXLib::CTexture::kType_direct
	
	Arguments : pMemory : BMP data, byteSize : Byte size of the BMP data

	Return value : bool
*********************************************************************************************************/
bool
CBmp::Read(
			TXLib::CTexture*		pTexture,						// Output CTexture pointer
	const	u8*						pMemory							// Input data pointer
)
{
	assert((pTexture != NULL ) && ( pMemory != NULL ));

	tFileHeader fileHeader;
	tInfoHeader infoHeader;

	// Header load
	if( !ReadBMPHEADER( &fileHeader, &infoHeader, pMemory ) ) return false;

	u16 width		= (u16)infoHeader.biWidth;
	u16 height		= (u16)infoHeader.biHeight;
	u32 nColor		= 0;


	//Create CBmp element corresponding to the bit depth
	if( infoHeader.biBitCount <= 8 )
	{ // Color index
		if( infoHeader.biBitCount == 1 )
		{
			pTexture->SetType( TXLib::CTexture::kType_palette4 );
			nColor = 2;						// Create palette with default number of colors
		}
		else if( infoHeader.biBitCount == 4 )
		{
			pTexture->SetType( TXLib::CTexture::kType_palette16 );

			if( infoHeader.biClrUsed )
			{
				nColor = infoHeader.biClrUsed;	// Create palette with the number of colors used
			}else{
				nColor = 16;					// Create palette with default number of colors
			}
		}
		else if( infoHeader.biBitCount == 8 )
		{
			pTexture->SetType( TXLib::CTexture::kType_palette256 );

			if( infoHeader.biClrUsed )
			{
				nColor = infoHeader.biClrUsed;	// Create palette with the number of colors used
			}else{
				nColor = 256;					// Create palette with the default number of colors
			}
		}
		else
		{
			return false;
		}

		ReadColorIndexImage	( &pTexture->GetColorIndexImage(), pMemory + fileHeader.bfOffBits, infoHeader.biBitCount, width, height ); // Create color index image
		ReadColorPalette( &pTexture->GetColorPalette(), pMemory + kDefine_fileHeaderByteSize + kDefine_infoHeaderByteSize, nColor );// Create color palette
	}
	else if( infoHeader.biBitCount == 24 )
	{ // direct
		pTexture->SetType( TXLib::CTexture::kType_direct );

		pTexture->GetAlphaImage().Create( width, height, 0xff );			// Create alpha
		ReadDirectColorImage(&pTexture->GetDirectColorImage(), pMemory + fileHeader.bfOffBits, width, height ); // Create direct color image
	}
	else
	{
		return false;
	}

	return true;
}

/********************************************************************************************************
	TXLibUtility::CBmp::Write()

	Write BMP data to memory

		If pByteSize exists, write the correct bite size

*********************************************************************************************************/
bool
CBmp::Write(
			u8*						pMemory,					// Output data pointer
			u32*					pByteSize,					// Number of bytes output
			TXLib::CTexture*		pTexture					// CTexture pointer
) const
{
	assert(( pMemory != NULL ) && (pTexture != NULL ));

	u16				bitCount;								// Bit depth
	u32				nColorUsed;								// Number of colors used in the color palette
	u16				width		= pTexture->GetWidth();		// Width
	u16				height		= pTexture->GetHeight();	// Height
	u32				dataOffset	= kDefine_fileHeaderByteSize + kDefine_infoHeaderByteSize;	// Image data offset

	TXLib::CTexture::tType	convertType		= GetConvertType( pTexture );			// CTexture format

	TXLib::CTexture texture;
	texture = *pTexture;		// Copy so that the original data does not get converted


	if( convertType != pTexture->GetType())
	{
		if( !texture.Convert( convertType )) return false;								// Convert
	}

	if( ( convertType == TXLib::CTexture::kType_palette16 ) || ( convertType == TXLib::CTexture::kType_palette256 ) )
	{
		bitCount	= ( convertType == TXLib::CTexture::kType_palette256 ) ? 8: 4;
		nColorUsed	= texture.GetColorPalette().GetNumber();

		WriteBMPHEADER( pMemory, pByteSize, bitCount, width, height, nColorUsed );			// Write header
		WriteColorPalette( pMemory + dataOffset, &texture.GetColorPalette() );				// Write palette
		WriteColorIndexImage( pMemory + dataOffset + (nColorUsed * 4), &texture.GetColorIndexImage(), bitCount);	// Write color index
	}
	else if( convertType == TXLib::CTexture::kType_direct )
	{
		WriteBMPHEADER( pMemory, pByteSize, 24, width, height, 0 );							// Write header
		WriteDirectColorImage( pMemory + dataOffset, &texture.GetDirectColorImage() );		// Write image
	}
	else
	{
		return false;
	}

	return true;
}

/********************************************************************************************************
	TXLibUtility::CBmp::GetConvertType()

	Get the conversion CTexture format when saving a BMP file.

			CTexture						BMP data
		TXLib::CTexture::kType_palette4		-> palette16
		TXLib::CTexture::kType_a5i3			-> palette16
		TXLib::CTexture::kType_palette16	-> palette16
		TXLib::CTexture::kType_a3i5			-> palette256
		TXLib::CTexture::kType_palette256	-> palette256
		TXLib::CTexture::kType_tex4x4		-> direct
		TXLib::CTexture::kType_direct		-> direct
*********************************************************************************************************/
TXLib::CTexture::tType
CBmp::GetConvertType(
			TXLib::CTexture*		pTexture						// CTexture pointer
) const
{
	assert( pTexture != NULL);

	TXLib::CTexture::tType		convertType;		// Conversion CTexture format

	switch( pTexture->GetType() )
	{
		case TXLib::CTexture::kType_palette4:
		case TXLib::CTexture::kType_a5i3:
		case TXLib::CTexture::kType_palette16:
			convertType	= TXLib::CTexture::kType_palette16;	break;
		case TXLib::CTexture::kType_a3i5:
		case TXLib::CTexture::kType_palette256:
			convertType	= TXLib::CTexture::kType_palette256;	break;
		case TXLib::CTexture::kType_tex4x4:
		case TXLib::CTexture::kType_direct:
			convertType	= TXLib::CTexture::kType_direct;	break;
		default:
			assert(false);
	}

	return convertType;
}

/********************************************************************************************************
	TXLibUtility::CBmp::GetDataByteSizeMax()

	Get the necessary file size for saving a BMP file.
*********************************************************************************************************/
bool
CBmp::GetDataByteSizeMax(
			u32*					pMemorySize,					// Memory size
			TXLib::CTexture*		pTexture						// CTexture Pointer
) const
{
	assert(( pMemorySize != NULL ) && (pTexture != NULL ));

	u16 width	= pTexture->GetWidth();
	u16 height	= pTexture->GetHeight();

	TXLib::CTexture::tType		convertType = GetConvertType( pTexture );		// Get the CTexture format at time of saving

	u32 byteSizeHeader	= kDefine_fileHeaderByteSize + kDefine_infoHeaderByteSize;
	u32 imageSize = 0;
	u32 memorySize = 0;

	if( convertType == TXLib::CTexture::kType_palette16 )
	{
		imageSize = GetNeedLine( 4, width ) * height;
		*pMemorySize = byteSizeHeader + imageSize + 16 * 4;
	}
	else if( convertType == TXLib::CTexture::kType_palette256 )
	{
		imageSize = GetNeedLine( 8, width ) * height;
		*pMemorySize = byteSizeHeader + imageSize + 256 * 4;
	}
	else if( convertType == TXLib::CTexture::kType_direct )
	{
		imageSize = GetNeedLine( 24, width ) * height;
		*pMemorySize = byteSizeHeader + imageSize;
	}
	else
	{
		return false;
	}

	return true;
}

} // namespace
