/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLibUtilityEx_Bg.h

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This class outputs CTexture data to BG data.

  $Log: TXLibUtilityEx_Bg.h,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Updated copyright. Made revision adjustments. 

  Revision 1.4	2004/10/27		mominoki
  Added bgAlignmentPaletteNumberFlg to Write() arguments. 
  Changed to allow selection of whether to output palette with 16 colors when 4 bit, and 256 colors when 8 bit, during BG output.  

  Revision 1.3	2004/10/22		mominoki
  Changed to output palette with 16 colors when 4 bit, and 256 colors when 8 bit, during BG output.  

  Revision 1.2	2004/07/28		mominoki
  
  Added offsetColorIndex to arguments of  Write(),MakeTextBg().
  Added functionality that offsets color index values with palette16, palette256, affine, palette256x16 

  Revision 1.1	2004/04/15		mominoki
  Fixed comments.

  Revision 1.0	2004/03/03		mominoki
  Entered initial version.

  $NoKeywords: $
 ***************************************************************************/

#include "stdafx.h"
#include <cstring>
#include "../include/TXLibUtilityEx_Bg.h"

namespace std{ using :: memset; }

namespace TXLibUtilityEx{

/*********************************************************************************************************
		private
*********************************************************************************************************/
/********************************************************************************************************
	TXLibUtilityEx::CBg::MakeTextBg()

	Create text BG
	Reorder a color index image to 8*8, and create character data and screen data.
*********************************************************************************************************/
bool
CBg::MakeTextBg(
		TXLib::CColorIndexImage*	pCharData,						// Character data
		u16*						pNumberChar,					// Total character count
		TXLib::CByteData*			pScreenData,					// Screen data
		tType						type,							// BG format
		bool						hFlip,							// There is H flip
		bool						vFlip,							// There is V flip
		bool						compressFlg,					// There is character compression
		u8							paletteNumber,					// Palette number
		u8							offsetColorIndex				// Color index offset value
) const
{
	assert((pCharData != NULL ) && (pScreenData != NULL ));
	if( !pCharData->IsAvailable() ) return false;

	u16 width	= pCharData->GetWidth();			// Height
	u16 height	= pCharData->GetHeight();			// Width

	// Create initial screen
	u16 numberBlock = (width *	height) / kDefine_block64;

	switch( type )
	{
		case kType_text_palette16:				// Text BG palette16
		case kType_text_palette256:				// Text BG palette256
		case kType_exAffine_palette256x16:		// Affine extension BG palette256 16 palette
			pScreenData->Create( 2, numberBlock, 0 );
			break;
		case kType_affine:						// Affine BG
			pScreenData->Create( 1, numberBlock, 0 );
			break;
		default:
			return false;
	}

	TXLib::CColorIndexImage		allCharData;	// All character data
	allCharData.Create( width, height);
	u16 numberAllCharData = 0;					// Number of all character data

	u8	charData[ kDefine_block64];				// One character data

	u32 block = 0;

	for (u16 y=0; y<height; y+=kDefine_Bg8)		// Line
	{
		for (u16 x=0; x<width; x+=kDefine_Bg8, block++ )	// Column
		{
			// Get one character data (8*8 pixels)
			u8		offset		= 0;
			tFlip	hvFlipData	= kType_notFlip;	// Screen data HV flip

			std::memset( charData, 0, sizeof(charData));	

			for( u16 block_y=0; block_y<kDefine_Bg8; block_y++)
			{
				for( u16 block_x=0; block_x<kDefine_Bg8; block_x++, offset++)
				{
					u8	colorIndex;
					u16 colorIndexBuf=0;
					
					pCharData->GetValue( &colorIndex, (x + block_x), (y + block_y) );	// Get the color index value

					if( type == kType_text_palette16)
					{
						colorIndexBuf = colorIndex + offsetColorIndex;	// Color index value + offset value

						if( colorIndexBuf > 15 )
						{
							colorIndexBuf = 15;
						}
					}
					else if( ( type == kType_text_palette256) || ( type == kType_exAffine_palette256x16) ||
							( type == kType_affine))
					{
						colorIndexBuf = colorIndex + offsetColorIndex;	// Color index value + offset value

						if( colorIndexBuf > 255 )
						{
							colorIndexBuf = 255;
						}
					}
					else
					{
						colorIndexBuf = colorIndex;	// Color index only
					}

					charData[ offset ] = (u8)colorIndexBuf;
				}
			}

			u16 newCharNumber = numberAllCharData;			// Character number (initial is the character number to add)

			if( compressFlg )
			{
				// Compare the color index of new character data and one character data.
				for( u32 block_i=0; block_i<numberAllCharData; block_i++)
				{
					bool	endFlg			= true;			// Character data comparison determination end flag
					u16		count			= 0;			// No flip	8x8 texel comparison counter
					u16		countHFlip		= 0;			// H flip		8x8 texel comparison counter
					u16		countVFlip		= 0;			// V flip		8x8 texel comparison counter
					u16		countHVFlip		= 0;			// HV flip	8x8 texel comparison counter

					// Character data comparison
					for( u8 index_y=0; index_y<kDefine_Bg8; index_y++)
					{
						u8 colorIndex;

						for( int index_x=0; index_x<kDefine_Bg8; index_x++)
						{
							u16 dataIndex		= index_y * kDefine_Bg8						+ index_x;

							// Get color index value from the newly created character data.			
							allCharData.GetColorIndexData().GetValue( &colorIndex, block_i * kDefine_block64 + dataIndex );

							u16 dataIndexHflip	= index_y * kDefine_Bg8						+ kDefine_Bg8 - index_x - 1;
							u16 dataIndexVflip	= (kDefine_Bg8 - index_y - 1) * kDefine_Bg8 + index_x;
							u16 dataIndexHVflip	= (kDefine_Bg8 - index_y - 1) * kDefine_Bg8 + kDefine_Bg8 - index_x - 1;

							// Compare color index values
							endFlg = true;		// End flag is on. (because if even one texel is different, the characters are not identical)

							if( colorIndex == charData[ dataIndex ] )								// No flip
							{
								count++;
								endFlg = false;
							}
							
							// Flip only enabled when not affine
							if( type != kType_affine )
							{
								if( ( colorIndex == charData[ dataIndexHflip ] ) && hFlip )				// There is HFlip
								{
									countHFlip++;
									endFlg = false;
								}

								if( ( colorIndex == charData[ dataIndexVflip ] ) && vFlip )				// There is VFlip
								{
									countVFlip++;
									endFlg = false;
								}

								if( ( colorIndex == charData[ dataIndexHVflip ] ) && hFlip && vFlip )	// There are HFlip, VFlip
								{
									countHVFlip++;
									endFlg = false;
								}
							}

							// Characters are not identical.
							if( endFlg )
							{
								index_x			= kDefine_Bg8;			// Set to the final texel and exit the loop.
								index_y			= kDefine_Bg8;
							}
						} // index_x
					} // index_y

					// Exit the loop if an identical character is found.
					if( count == kDefine_block64 )
					{
						newCharNumber	= block_i;		// Set the current character number to the character.
						hvFlipData		= kType_notFlip;
						break;
					}
					else if( countHFlip == kDefine_block64 )
					{
						newCharNumber	= block_i;		// Set the current character number to the character.
						hvFlipData		= kType_hFlip;
						break;
					}
					else if( countVFlip == kDefine_block64 )
					{
						newCharNumber	= block_i;		// Set the current character number to the character.
						hvFlipData		= kType_vFlip;
						break;
					}
					else if( countHVFlip == kDefine_block64 )
					{
						newCharNumber	= block_i;		// Set the current character number to the character.
						hvFlipData		= kType_hvFlip;
						break;
					}
				}
			} // compressFlg(character compression)


			// Add if not an identical character
			if( newCharNumber == numberAllCharData )
			{
				for( u16 dataIndex=0; dataIndex<kDefine_block64; dataIndex++)
				{
					u32 dataOffset	= newCharNumber * kDefine_block64 + dataIndex;
					allCharData.GetColorIndexData().SetValue( charData[dataIndex], dataOffset );
				}
				numberAllCharData++;
			}

			// Create screen data
			u16 data = newCharNumber;

			// Affine is 8-bit screen data
			if( type == kType_text_palette16 )
			{
				data |= ( (u16)hvFlipData << kDefine_flipShift );		// Flip
				data |= ( (u16)paletteNumber << kDefine_paletteShift );	// Palette number
				pScreenData->SetValue( data, block );					// Set
			}	
			else if( type == kType_text_palette256 )
			{
				data |= ( (u16)hvFlipData << kDefine_flipShift );		// Flip
				pScreenData->SetValue( data, block );					// Set
			}
			else if( type == kType_affine )
			{
				pScreenData->SetValue( data, block );					// Set
			}
			else if( type == kType_exAffine_palette256x16 )
			{
				data |= ( (u16)hvFlipData << kDefine_flipShift );		// Flip
				data |= ( (u16)paletteNumber << kDefine_paletteShift );	// Palette number
				pScreenData->SetValue( data, block );					// Set
			}
			else
			{
				return false;
			}

		}
	}
	// Copy
	*pCharData		= allCharData;			// Character data
	*pNumberChar	= numberAllCharData;	//Total character count

	return true;
}

/********************************************************************************************************
	TXLibUtilityEx::CBg::MakeBmpScreen()

	Create Bmp screen
*********************************************************************************************************/
void
CBg::MakeBmpScreen(
			TXLib::CByteData*			pScreen,							// Screen
	const	TXLib::CDirectColorImage*	pDirectColorImage,					// Direct color image
	const	TXLib::CAlphaImage*			pAlphaImage							// Alpha image
) const
{
	assert( ( pScreen != NULL ) && ( pDirectColorImage != NULL ) && ( pAlphaImage != NULL) );
	
	u16 width	= pDirectColorImage->GetWidth();
	u16 height	= pDirectColorImage->GetHeight();

	u8	red, green, blue, alpha;
	u16 data_16bit;
	u32 index=0;
	for( u16 y=0; y<height; y++)
	{
		for( u16 x=0; x<width; x++, index++)
		{
			pDirectColorImage->GetValue( &red, &green, &blue, x, y	); // Get image color value

			// Convert 8bit->5bit
			red >>= 3;
			green >>= 3;
			blue >>= 3;

			pAlphaImage->GetValue( &alpha, x, y ); // Get alpha value

			if(alpha)
			{
				alpha = 1; // Convert alpha 8bit -> 1bit (if alpha exists, "1")
			}
			data_16bit =  TLB_DATA_1555( alpha, blue, green, red); // Reorder R,G,B order, and create palette color data.

			pScreen->SetValue( data_16bit, index );
		}
	}
}


/********************************************************************************************************
	TXLibUtilityEx::CBg::WriteChar()

	Output character data
*********************************************************************************************************/
void
CBg::WriteChar(
			u8*							pCharData,						// Character data pointer
	const	TXLib::CColorIndexImage*	pColorIndexImage,				// Color index
			u8							bitCount						// Bit depth
) const
{
	assert( ( pCharData != NULL) && ( pColorIndexImage != NULL ));

	u16 width	= pColorIndexImage->GetWidth();
	u16 height	= pColorIndexImage->GetHeight();

	u8	index;
	u16	data_16bit;
	u32 offset;
	for(u16 y=0; y<height; y++)
	{
		for(u16 x=0; x<width; x++)
		{
			pColorIndexImage->GetValue( &index, x, y ); // Get color index value

			switch( bitCount )
			{
				case 4:
					offset = ( ( y * width / 4 ) + x / 4 ) * 2;
					data_16bit = ( (u16)index << ( ( x % 4 ) * 4 ) );
					*(u16*)( pCharData + offset ) |= data_16bit;
					break;
				case 8:
					offset = ( ( y * width / 2 ) + x / 2 ) * 2;
					data_16bit = ( (u16)index << ( ( x % 2 ) * 8 ) );
					*(u16*)( pCharData + offset ) |= data_16bit;
					break;
				default:
					assert(false);
			}
		}
	}
}

/********************************************************************************************************
	TXLibUtilityEx::CBg::WritePalette()

	Output palette data
*********************************************************************************************************/
void
CBg::WritePalette(
				u8*							pPaletteData,					// Palette data pointer
				u32*						pPaletteDataByteSize,			// Palette data byte size
		const	TXLib::CColorPalette*		pColorPalette					// Color palette
) const
{
	assert( ( pPaletteData != NULL) && ( pColorPalette != NULL ) );

	u16 nColor = pColorPalette->GetNumber();
	*pPaletteDataByteSize = nColor * 2;			// Align byte size to palette count

	u8	red, green, blue;
	u16 data_16bit;
	u32 offset;
	for( u32 number=0; number<nColor; number++ )
	{
		pColorPalette->GetValue( &red, &green, &blue, number ); // Get color value

		// Convert 8bit->5bit
		red >>= 3;
		green >>= 3;
		blue >>= 3;

		data_16bit =  TLB_DATA_1555( 0, blue, green, red); // Reorder R,G,B order and create palette color data

		offset = number * 2;
		*(u16*)( pPaletteData + offset ) = data_16bit; // Set in memory
	}
}

/********************************************************************************************************
	TXLibUtilityEx::CBg::WriteScreen()

	Output screen data
*********************************************************************************************************/
void
CBg::WriteScreen(
				u8*							pScreenData,					// Character data pointer
				u32*						pScreenDataByteSize,			// Character data byte size
		const	TXLib::CByteData*			pScreen							// Screen
) const
{
	assert( ( pScreenData != NULL) && ( pScreen != NULL) );

	*pScreenDataByteSize = pScreen->GetByteCount() * pScreen->GetNumber();

	u32 data_32bit;
	for( u32 number=0; number<pScreen->GetNumber(); number++ )
	{
		pScreen->GetValue( &data_32bit, number ); // Get screen value

		if( pScreen->GetByteCount() == 1 )
		{
			*( pScreenData + number ) = (u8)data_32bit; // Set in memory
		}
		else if( pScreen->GetByteCount() == 2 )
		{
			*(u16*)( pScreenData + number * 2 ) = (u16)data_32bit; // Set in memory
		}
	}
}

/*********************************************************************************************************
		public
*********************************************************************************************************/
/********************************************************************************************************
	TXLibUtilityEx::CBg::Write()

	Output BG data
*********************************************************************************************************/
bool
CBg::Write(
		tMemory*					pBgMemory,						// BG memory
		TXLib::CTexture*			pTexture,						// CTexture pointer (input)
		tType						bgType,							// BG format
		bool						hFlip,							// There is H flip
		bool						vFlip,							// There is V flip
		bool						compressFlg,					// There is character compression
		u8							paletteNumber,					// Palette number
		bool						bgAlignmentPaletteNumberFlg,	// Palette number adjustment flag 
		u8							offsetColorIndex				// Color index offset value
) const
{
	assert(( pBgMemory != NULL ) && (pTexture != NULL ) );

	u16 width	= pTexture->GetWidth();
	u16 height	= pTexture->GetHeight();


	// Clear memory
	std::memset(pBgMemory->pCharData,		0, pBgMemory->charDataByteSize);
	std::memset(pBgMemory->pPaletteData,	0, pBgMemory->paletteDataByteSize);
	std::memset(pBgMemory->pScreenData,		0, pBgMemory->screenDataByteSize);

	TXLib::CTexture texture;
	texture = *pTexture;		// Copy so that you do not convert original data.

	TXLib::CByteData	screenData;		// Screen data

	switch( bgType)
	{
		case	kType_text_palette16:			// Text BG palette16
		case	kType_text_palette256:			// Text BG palette256
		case	kType_affine:					// Affine BG
		case	kType_exAffine_palette256x16:	// Affine extension BG palette256 16 palette
			// Clear, because it changes character count
			pBgMemory->charDataByteSize = 0;

			// Error determination
			if((width % 8 ) || ( height % 8 ) ) return false;		// Check size

			// Convert
			u8 bitCount;
			if( bgType == kType_text_palette16 )
			{
				if( texture.GetType() != TXLib::CTexture::kType_palette16) return false;
				bitCount = 4;
			}
			else
			{
				if( texture.GetType() != TXLib::CTexture::kType_palette256) return false;
				bitCount = 8;
			}

			u16 numberChar;
			if( !MakeTextBg(&texture.GetColorIndexImage(),
							&numberChar,
							&screenData,
							bgType,
							hFlip,
							vFlip,
							compressFlg,
							paletteNumber,
							offsetColorIndex	))
			{
				return false; 						// Create text BG data
			}

			// Determine whether character count is over
			if( ( bgType == kType_text_palette16 ) || ( bgType == kType_text_palette256 ) ||
				( bgType == kType_exAffine_palette256x16 ))
			{
				if( numberChar > kDefine_textCharNumberMax ) return false;
			}
			else if( bgType == kType_affine )
			{
				if( numberChar > kDefine_affineCharNumberMax ) return false;
			}

			pBgMemory->charDataByteSize = numberChar * kDefine_block64 / ( 8 / bitCount);
			WriteChar(		pBgMemory->pCharData,		&texture.GetColorIndexImage(), bitCount );

			if( bgAlignmentPaletteNumberFlg )
			{
				if( bitCount == 4 )
				{
					texture.GetColorPalette().ChangeNumber(16);
				}
				else if( bitCount == 8)
				{
					texture.GetColorPalette().ChangeNumber(256);
				}
			}

			WritePalette(	pBgMemory->pPaletteData,	&pBgMemory->paletteDataByteSize, &texture.GetColorPalette() );
			WriteScreen(	pBgMemory->pScreenData,		&pBgMemory->screenDataByteSize,	&screenData );
			break;
		case	kType_exAffine_palette256Bmp:		// Affine extension BG palette256 BMP
			if( texture.GetType() != TXLib::CTexture::kType_palette256) return false;

			WritePalette(	pBgMemory->pPaletteData,	&pBgMemory->paletteDataByteSize, &texture.GetColorPalette() );
			WriteChar(		pBgMemory->pScreenData,		&texture.GetColorIndexImage(),	8 );
			break;
		case	kType_exAffine_direct:	// Affine extension BG DIRECT BMP
			if( texture.GetType() != TXLib::CTexture::kType_direct) return false;

			screenData.Create( 2, (width * height), 0 );								// Create screen
			if( !texture.Convert( TXLib::CTexture::kType_direct )) return false;				// Convert
			MakeBmpScreen( &screenData, &texture.GetDirectColorImage(), &texture.GetAlphaImage() );	// Create Screen data
			WriteScreen(	pBgMemory->pScreenData,		&pBgMemory->screenDataByteSize,	&screenData );
			break;
		case	kType_big256Bmp:				// Large screen BMP
			if( texture.GetType() != TXLib::CTexture::kType_palette256) return false;

			WritePalette(	pBgMemory->pPaletteData,	&pBgMemory->paletteDataByteSize, &texture.GetColorPalette() );
			WriteChar(		pBgMemory->pScreenData,		&texture.GetColorIndexImage(),	8 );
			break;
		default:
			return false;
	}

	return true;
}

/********************************************************************************************************
	TXLibUtilityEx::CBg::GetDataByteSizeMax()

	Get memory size required for BG data output
*********************************************************************************************************/
bool
CBg::GetDataByteSizeMax(
		u32*						pCharDataByteSize,				// Character data size
		u32*						pPaletteDataByteSize,			// Palette data size
		u32*						pScreenDataByteSize,			// Screen data size
		u16							width,							// Width
		u16							height,							// Height
		tType						bgType							// BG format to output
) const
{
	assert(( pCharDataByteSize != NULL ) && (pPaletteDataByteSize != NULL ) && ( pScreenDataByteSize != NULL) );

	switch( bgType)
	{
		case	kType_text_palette16:			// Text BG palette16
			*pCharDataByteSize		= width * height / 2;
			*pPaletteDataByteSize	= 256 * 2;
			*pScreenDataByteSize	= (width / 8) * (height / 8) * 2;
			break;
		case	kType_text_palette256:			// Text BG palette256
		case	kType_exAffine_palette256x16:	// Affine extension BG palette256 16 palette
			*pCharDataByteSize		= width * height;
			*pPaletteDataByteSize	= 256 * 2;
			*pScreenDataByteSize	= (width / 8) * (height / 8) * 2;
			break;
		case	kType_affine:					// Affine BG
			*pCharDataByteSize		= width * height;
			*pPaletteDataByteSize	= 256 * 2;
			*pScreenDataByteSize	= (width / 8) * (height / 8);
			break;
		case	kType_exAffine_palette256Bmp:	// Affine extension BG palette256 BMP
		case	kType_big256Bmp:				// Large screen BMP
			*pCharDataByteSize		= 0;
			*pPaletteDataByteSize	= 256 * 2;
			*pScreenDataByteSize	= width * height;
			break;
		case	kType_exAffine_direct:	// Affine extension BG DIRECT BMP
			*pCharDataByteSize		= 0;
			*pPaletteDataByteSize	= 0;
			*pScreenDataByteSize	= width * height * 2;
			break;
		default:
			return false;
	}

	return true;
}

} // namespace