/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    MyColorProcessor.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 This class conducts color processing. It is a function customized only to extract 0-4 colors from the block
  during conversion to tex4x4. All other functions are left to CDefaultColorProcessor.

  $Log: MyColorProcessor.cpp,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Corrected comment

  Revision 1.0  2004/03/03		mominoki
  Initial registration

  $NoKeywords: $
 ***************************************************************************/
#include "stdafx.h"
#include <TXLib.h>
#include "MyColorProcessor.h"

/*********************************************************************************************************
	TXLib::CMyColorProcessor::MakePalette_Tex4x4

	Creates color palettes (0-4 colors) of high, medium, and low brightness.	When linear interpolation is specified, only 2 colors are taken out.
*********************************************************************************************************/
bool
CMyColorProcessor::MakePalette_Tex4x4( 
				TXLib::CColorPalette*		pNewColorPalette,				// Newly-created palette
		const	TXLib::CColorPalette* 		pOldColorPalette,				// Current color palette
				bool						transparencyFlg,				// Transparent color mode flag
		bool	linearFlg													// Linear interpolation flag
)
{
	assert( (pOldColorPalette != NULL) && (pNewColorPalette != NULL ) );

	pNewColorPalette->ChangeNumber(4);	// Change the palette count to the maximum (4 colors)

	TXLib::CColorPalette	colorMemory;
	colorMemory = *pOldColorPalette;						// Copy the current palette for processing

	colorMemory.QuickSortTopDown( true );					// Arrange in descending order of brightness

	u32 hiYNumber, lowYNumber;								// Index of brightest and least bright color
	hiYNumber = 0;
	lowYNumber = colorMemory.GetNumber() - 1;

	u8 red0, green0, blue0;
	u8 red1, green1, blue1;
	u8 red2, green2, blue2;
	u8 red3, green3, blue3;

	// The brightest color is set to 0, the least bright to 1
	colorMemory.GetValue( &red0, &green0, &blue0, hiYNumber );
	pNewColorPalette->SetValue( red0, green0, blue0, 0 );

	colorMemory.GetValue( &red1, &green1, &blue1, lowYNumber );
	pNewColorPalette->SetValue( red1, green1, blue1, 1 );

	if( linearFlg )
	{
		// If there is linear interpolation, there are only 2 colors, so set the number of palettes to 2 colors and extract the function.
		pNewColorPalette->ChangeNumber(2);
	}
	else
	{
		//  If there is no linear interpolation, for transparent color mode set 1 color remaining, and for 4-color mode set colors remaining


		if( transparencyFlg )							// Transparent color mode
		{
			red2	=( red0		+ red1 )	/ 2;
			green2	=( green0	+ green1 )	/ 2;
			blue2	=( blue0	+ blue1 )	/ 2;

			pNewColorPalette->ChangeNumber(3);
			pNewColorPalette->SetValue( red2, green2, blue2, 2 );
		}
		else											// 4-color mode
		{
			red2	=( red0		+ red1 )	/ 3;
			green2	=( green0	+ green1 )	/ 3;
			blue2	=( blue0	+ blue1 )	/ 3;

			red3	=(( red0	+ red1 )		* 2) / 3;
			green3	=(( green0	+ green1 )		* 2) / 3;
			blue3	=(( blue0	+ blue1 )		* 2) / 3;

			pNewColorPalette->SetValue( red2, green2, blue2, 2 );
			pNewColorPalette->SetValue( red3, green3, blue3, 3 );
		}
	}

	return true;
}


/*********************************************************************************************************
	TXLib::CMyColorProcessor::GetTex4x4Palette

	Get a tex4x4 palette from one block.
*********************************************************************************************************/
bool
CMyColorProcessor::Get4Color_Tex4x4(
			TXLib::CColorPalette*		pColorPalette,					// Color palette
			bool						transparencyFlg,				// Transparent color mode flag
	const	TXLib::CDirectColorImage*	pBlockDirectColorImage,			// 1 block image
	const	TXLib::CAlphaImage*			pBlockAlphaImage,				// 1 block alpha
			tAlgorithm					algorithm						// Algorithm
) 
{
	// Error processing
	assert( (pColorPalette != NULL ) && ( pBlockDirectColorImage != NULL ) && ( pBlockAlphaImage != NULL) );
	assert( pColorPalette->IsAvailable() && pBlockDirectColorImage->IsAvailable() );
	assert( ( pBlockDirectColorImage->GetWidth() == TLB_4 ) && ( pBlockDirectColorImage->GetHeight() == TLB_4 ) );
	assert( ( pBlockAlphaImage->GetWidth() == TLB_4 ) && ( pBlockAlphaImage->GetHeight() == TLB_4 ) );

	TXLib::CColorPalette		tex4x4Palette;		// tex4x4 palette

	tex4x4Palette.Create( (TLB_4 * TLB_4), 0, 0, 0 );		// Secure color memory for one block (16 colors)

	// Put the alpha extracted image from a 1 block image into color memory. (Because transparent colors cannot be put into color subtraction algorithms)
	CreateColorMemory( &tex4x4Palette, pBlockDirectColorImage, pBlockAlphaImage, 0x80 );

	tex4x4Palette.CommonLowerBit(5);						// Pseudo convert an 8-bit color value to 5 bits.

	TXLib::CColorPalette	tex4x4PaletteBuf = tex4x4Palette;
	tex4x4PaletteBuf.CutCommonColor();						// Cut shared colors

	if( tex4x4PaletteBuf.GetNumber() < 3 )
	{
		// Put into palette as-is if 4 colors or less
		if(!SetPalette_Tex4x4( pColorPalette, &tex4x4PaletteBuf, transparencyFlg )) return false;
	}
	// Subtractive color processing if 5 colors or more
	else
	{
		if( algorithm == 0 )
		{
			if( !MakePalette_Tex4x4( pColorPalette, &tex4x4Palette, transparencyFlg, false ) ) return false;	// No linear interpolation
		}
		else
		{
			if( !MakePalette_Tex4x4( pColorPalette, &tex4x4Palette, transparencyFlg, true ) ) return false;		// Linear interpolation
		}
	}

	return true;
}

/*********************************************************************************************************
	TXLib::CMyColorProcessor::ConvertToTex4x4

	Convert from direct to tex4x4
	Called from CTexture::ChangeDirectToTex4x4()
*********************************************************************************************************/
bool
CMyColorProcessor::ConvertToTex4x4(
			TXLib::CTexture*					pTexture					// Texture
)
{
	assert( pTexture );

	TXLib::CAlphaImage*				pAlphaImage				= &pTexture->GetAlphaImage();
	TXLib::CByteData*				pColorPaletteAddress	= &pTexture->GetColorPaletteAddress();			
	TXLib::CColorIndexImage*		pColorIndexImage		= &pTexture->GetColorIndexImage();
	TXLib::CColorPalette*			pColorPalette			= &pTexture->GetColorPalette();
	TXLib::CData*					pColorPaletteAttribute	= &pTexture->GetColorPaletteAttribute();
	TXLib::CDirectColorImage*		pDirectColorImage		= &pTexture->GetDirectColorImage();

	assert( pColorIndexImage->IsAvailable()		&& pColorPalette->IsAvailable() && 
			pDirectColorImage->IsAvailable()	&& pAlphaImage->IsAvailable() &&
			pColorPaletteAddress->IsAvailable() && pColorPaletteAttribute->IsAvailable());

	u16 width  = pTexture->GetWidth();
	u16 height = pTexture->GetHeight();

	u16	paletteAddress	= 0;	// Palette address
	u32	block			= 0;	// Number of block
	u32 nNewColor		= 0;	// Total number of colors in palettes

	for (u16 y=0; y<height; y+=TLB_4)				// Row
	{
		for (u16 x=0; x<width; x+=TLB_4, block++)	// Column
		{
			//--------------------------------------------------------------
			//	Create a one block direct color image
			//--------------------------------------------------------------
			TXLib::CDirectColorImage	blockDirectColorImage;
			blockDirectColorImage.Create( TLB_4, TLB_4, 0, 0, 0 );		// Create 1 block image

			MakeBlockDirectColorImage_Tex4x4(
						&blockDirectColorImage,
						pDirectColorImage,
						x,
						y);

			//--------------------------------------------------------------
			//	Create a one block alpha image
			//--------------------------------------------------------------
			TXLib::CAlphaImage		blockAlphaImage;
			blockAlphaImage.Create( TLB_4, TLB_4, 0xff );			// Default is opaque

			bool fourColorFlg = false;		// 4 color mode flag. Always handle as 4 color mode if there is no transparent color in the block.

			if( GetConvertFactor().transparencyFlg )
			{
				if( GetConvertFactor().alphaColorCutFlg )
				{
					// Use alpha to create alpha. For alpha values less than the border value, set the alpha value to 0.
					fourColorFlg = MakeBlockAlphaImageUseAlpha_Tex4x4(
							&blockAlphaImage,
							pAlphaImage,
							x,
							y,
							0x80);
				}
				else
				{
					// Use transparent color to create alpha. If the image color value is the same as the transparent color, set the alpha value to 0.
					fourColorFlg = MakeBlockAlphaImageUseTransparentColor_Tex4x4(
							&blockAlphaImage,
							&blockDirectColorImage,
							GetConvertFactor().transparentColor);
				}
			}

			//--------------------------------------------------------------
			//	Get tex4x4 palette (0-4 colors) from one block.
			//--------------------------------------------------------------
			TXLib::CColorPalette	fourColorPalette;

			fourColorPalette.Create( 4, 0, 0, 0 );

			if( !Get4Color_Tex4x4(
						&fourColorPalette,
						fourColorFlg,
						&blockDirectColorImage,
						&blockAlphaImage,
						GetConvertFactor().algorithm
						)) return false;

			//--------------------------------------------------------------
			//	Create Color Palette Attributes
			//--------------------------------------------------------------
			u8 attribute = 0;
			
			if( fourColorPalette.GetNumber() < 3 )
			{
				attribute |= 1;			// Linear interpolation
			}

			if( fourColorFlg )
			{
				attribute |= 2;			// 4 colors
			}

			pColorPaletteAttribute->SetValue( attribute, block  );


			//--------------------------------------------------------------
			//	Create color palette and color palette address
			//--------------------------------------------------------------
			u8	nCompareColor	= fourColorPalette.GetNumber();	// The number of colors in the tex4x4 palette.

			if( nCompareColor == 0 )
			{
				// If all are transparent colors, the palette address is "0".
				pColorPaletteAddress->SetValue( 0, block );
			}
			else
			{
				// Search for an identical tex4x4 palette.
				int	paletteIndex = -1;

				if( GetConvertFactor().compressFlg )
				{
					paletteIndex = FindNearPalette(
								&fourColorPalette,
								pColorPalette,
								nNewColor,
								GetConvertFactor().commonColorRange.red		<< 3,
								GetConvertFactor().commonColorRange.green	<< 3,
								GetConvertFactor().commonColorRange.blue	<< 3
								);
				}

				if( paletteIndex == -1 )
				{
					// When there are no idential tex4x4 palettes.
					u8	red, green, blue;								// tex4x4 palette color elements

					// Set new palette address
					pColorPaletteAddress->SetValue( (nNewColor >> 1 ), block );

					// Add tex4x4 palette
					for( u8 n=0; n<nCompareColor; n++ )
					{
						fourColorPalette.GetValue(  &red, &green, &blue , n );	// Get tex4x4 palette color
						pColorPalette->SetValue( red, green, blue, nNewColor );	// Add to palette
						nNewColor++;
					}

					// Aligh number of palettes to an even number
					if( ( nCompareColor == 1 ) || ( nCompareColor == 3 ) )
					{
						pColorPalette->SetValue( 0, 0, 0, nNewColor );	// Add dummy to palette
						nNewColor++;
					}
				}
				else
				{
					// If found, set an existing palette address.
					pColorPaletteAddress->SetValue( (paletteIndex >> 1), block );
				}
			}


			//--------------------------------------------------------------
			//	Create color index
			//--------------------------------------------------------------
			if( fourColorPalette.GetNumber() == 2 )
			{
				ConvertLinearColorPalette_Tex4x4( &fourColorPalette, fourColorFlg );	// Create a tex4x4 linearly interpolated palette
			}

			for( u32 block_y = 0; block_y<TLB_4; block_y++)
			{
				for( u32 block_x = 0; block_x<TLB_4; block_x++)
				{
					int index;
					u8 alpha;

					blockAlphaImage.GetValue( &alpha, block_x, block_y  );			// Get alpha value

					if( alpha < 0x80 )
					{
						index = 3;			// Transparent color
					}
					else
					{
						u8 red, green, blue;

						blockDirectColorImage.GetValue( &red, &green, &blue, block_x, block_y  );	// Get image color value
						index = FindNearColor( &fourColorPalette, red, green, blue, 3 );			// Search palette from color index
						if( index == -1) return false;
					}
					pColorIndexImage->SetValue( (u8)index, (x + block_x), (y + block_y)  );	// Put index value to color index
				}
			}
		}
	}

	// Change the number of palette colors from the default to the actual number.
	pColorPalette->ChangeNumber( nNewColor );

	// If the number of palette colors exceeds the palette memory, compress the number of colors until they can be contained in the palette memory.
	if( nNewColor > ( TLB_PALETTEADDRESS_MAX * 2 ) )
	{
		u8	commonRed	= GetConvertFactor().commonColorRange.red;
		u8	commonGreen = GetConvertFactor().commonColorRange.green;
		u8	commonBlue	= GetConvertFactor().commonColorRange.blue;
		u8	escapeEndlessRoopCount	= 0;	// Infinite loop avoidance counter
		u8	compressSpeedFactor		= 8;	// Compression speed (Coefficient that expands the common range value. Only use it if the number of colors is excessive.)

		while( pColorPalette->GetNumber() > ( TLB_PALETTEADDRESS_MAX * 2 ) )
		{
			if( commonRed < ( 255 - compressSpeedFactor) )		// So that it does not exceed the maximum value (255).
			{
				commonRed	+= compressSpeedFactor;
			}
			if( commonGreen < ( 255 - compressSpeedFactor) )
			{
				commonGreen	+= compressSpeedFactor;
			}
			if( commonBlue < ( 255 - compressSpeedFactor) )
			{
				commonBlue	+= compressSpeedFactor;
			}

			if(!CompressColorNumber_Tex4x4( 
						pColorPalette,
						pColorPaletteAddress,
						pColorPaletteAttribute,
						commonRed << 3,
						commonGreen << 3,
						commonBlue << 3
						)) return false;

			// Avoid infinite loop (just in case)
			escapeEndlessRoopCount++;
			if( escapeEndlessRoopCount >30 )
			{
				return false;
			}
		}
	}

	return true;
}
