/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    ntexconv.cpp

	Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

	Based on the options, this tool converts BMP or TGA files to NTF and then outputs.

	texconv input image file [options]

	(Examples)
	texconv source.bmp -o destination.c -f tex4x4 -a3 -tc=255,0,0 -cp=2,2,2
	texconv source.bmp -d after.bmp -f tex4x4 -a3 -ta -ncp
	texconv source.bmp -s script.txt -v -tv
	texconv source.bmp -o destination.c -bg -f palette16 -bgp 4

  $Log: ntexconv.cpp,v $
  Revision 1.13  2005/06/23 08:14:10  mominoki
  Version changed to 1.5.0

  Revision 1.12  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.6	2004/10/28		mominoki
  When using BG output by script, script data number was not counted correctly. We fixed this bug. 
  When converting from direct to a3i5/a5i3, we added the ability to select as an option -ta, which was previously always specified.
  
  Revision 1.5	2004/10/27		mominoki
  Added option for palette number alignment (-bgnap) during BG output.
  When specified, only the number of used palettes is output.
  When not specified, if 16 colors, output with 16 colors; if 256 colors, output with 256 colors. 

  Revision 1.4	2004/09/17		mominoki
  For error output, changed printf() in afterErrorProcess() to fprintf().

  Revision 1.3	2004/07/28		mominoki
  Added color index offset functionality for BG output when BG format is 
  palette16, palette256, affine or palette256x16.

  Revision 1.2	2004/06/25		mominoki
  Fixed the bug that caused incorrect color subtraction when -ta is not specified 
  during a conversion from direct to a3i5, a5i3. 
  Changed the program so that -ta is always specified inside the program. 

  Revision 1.1	2004/04/15		mominoki
  Revised comments.

  Revision 1.0	2004/03/03		mominoki
  Registration of Initial release. 

  $NoKeywords: $
 ***************************************************************************/
#include "stdafx.h"
#include <cstdio>

#ifdef TLB_NTEXCONV_PRINT_TIME
#include <ctime>
#endif

#include <TXLib.h>
#include "../TXLibUtility/include/TXLibUtility.h"
#include "../TXLibUtilityEx/include/FileTextureViewer.h"
#include "../TXLibUtilityEx/include/TXLibUtilityEx_SaveBg.h"

#include "MyCUtility.h"
#include "inputData.h"
#include "ntexconv.h"

/****************************************************************************
	printHelpView()

	Display help screen
 ***************************************************************************/
static void printHelpView( void )
{
	printf(
		"option List\n"
		"---------------  options that can only be used outside of the script  -----------------\n"
		"	-v                    Displays detailed conversion status\n"
		"	-h                    Displays help screen and terminates\n"
		"	-s  [ path ]          Converts according to the content of the specified script\n"
		"	-tv                   Outputs data for TextureViewer\n"
		"\n--------------- options that can be used inside or outside of the script ---------------\n"
		"	-o  [ path ]          Specifies NTF output path name\n"
		"	-no                   Does not output NTF\n"
		"	-b                    Outputs NTF in binary format\n"
		"	-d  [ path ]          Outputs option file. Specify path\n"
		"	-p  [ path ]          Specifies palette used when converted to palette format \n"
		"	                      (image file path)\n"
		"	-f  [ format ]        Specifies texture format (palette4 ,a5i3, palette16, a3i5,"
		"					      palette256, direct, or tex4x4)\n"
		"	-nal			      Does not alignment NTF size\n"
		"\n	( palette4, palette16, palette256, a3i5, and a5i3 options )\n"
		"	-ta                   Creates palette number 0 using alpha\n"
		"	-tc=r,g,b             Specifies transparent color (r ,g ,b) and creates palette number 0\n"
		"\n	( tex4x4 options )\n"
		"	-a0                   Conversion algorithm (Fast, without linear interpolation)\n"
		"	-a1                   Conversion algorithm (Fast, with linear interpolation)\n"
		"	-a2                   Conversion algorithm (Round robin, without linear interpolation)\n"
		"	-a3                   Conversion algorithm (Round robin, with linear interpolation)\n"
		"	-ta                   Transparent color mode using alpha\n"
		"	-tc=r,g,b             Transparent color mode using a transparent color (r, g, b)\n"
		"	-cp=r,g,b             Compresses the number of palettes (specify color range in r, g, b)\n"
#ifdef USE_BG
		"	-ncp                  Does not compress the number of palettes\n"
		"\n	( BG options )\n"
		"	-bg                   Outputs BG\n"
		"	-bgb                  Outputs BG in binary format\n"
		"	-bgo [ path ]         Specifies BG output file name\n"
		"	-bgf [ format ]       Specifies BG format (palette16, palette256, affine, direct,\n"
		"				palette256x16, palette256Bmp, big256Bmp)\n"
		"	-bgp [ number ]       Specifies Palette Number (when converting to palette16\n"
		"				or palette256x16)\n"
		"	-bgnap                No Alignment Palette Number (when converting to palette16\n"
		"				or palette256 or palette256x16)\n"
		"	-bgnv                 No V-Flip (when converting to palette16 or palette256 or \n"
		"				palette256x16)\n"
		"	-bgnh                 No H-Flip (when converting to palette16 or palette256 or \n"
		"				palette256x16)\n"
		"	-bgnc                 Does not compress number of character(when converting to \n"
		"				palette16 or palette256 or palette256x16\n"
		"	-bgof                 Offset image of character(when converting to \n"
		"				palette16 or palette256 or affine or palette256x16\n"
		"	-ta                   Creates palette number 0 using alpha\n"
		"	-tc=r,g,b             Specifies transparent color (r ,g ,b) and creates palette number 0\n\n");
#else
		"	-ncp                  Does not compress number of palettes\n\n");
#endif
}

/****************************************************************************
	GetOptionInputDataNumber()

	Gets the number of the option input
 ***************************************************************************/
static tErrorStatus
GetOptionInputDataNumber(
	int*			pOption,
	int				argc,
	_TCHAR*			argv[]
)
{
	assert( pOption != NULL );

	int				notCount = 0;		// Number with no "-" specified
	char*			pChar;

	for( int number=1; number< argc; )
	{
		if( (*(pChar = argv[number])) == '-' )
		{
			pChar++;

			if( strcmp(	pChar,		&optionStr[ kOption_View ][0]			) == 0)	{ pOption[ kOption_View ]			= number;}
			else if( strcmp( pChar, &optionStr[ kOption_Help ][0]			) == 0)	{ pOption[ kOption_Help ]			= number;}
			else if( strcmp( pChar, &optionStr[ kOption_Script ][0]			) == 0)	{ pOption[ kOption_Script ]			= number++;}
			else if( strncmp( pChar, &optionStr[ kOption_TextureViewer ][0], 3		) == 0)	{ pOption[ kOption_TextureViewer ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_NtfPathName ][0]	) == 0)	{ pOption[ kOption_NtfPathName ]	= number++;}
			else if( strcmp( pChar, &optionStr[ kOption_NotNtf ][0]			) == 0)	{ pOption[ kOption_NotNtf ]			= number;}
			else if( strcmp( pChar, &optionStr[ kOption_NtfBinary ][0]		) == 0)	{ pOption[ kOption_NtfBinary ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_OptionPathName ][0]	) == 0)	{ pOption[ kOption_OptionPathName ]	= number++;}
			else if( strcmp( pChar, &optionStr[ kOption_PalettePathName ][0]) == 0)	{ pOption[ kOption_PalettePathName ]= number++;}
			else if( strcmp( pChar, &optionStr[ kOption_TextureFormat ][0]	) == 0)	{ pOption[ kOption_TextureFormat ]	= number++;}
			else if( strcmp( pChar, &optionStr[ kOption_NotAlignmentNtfSize ][0]) == 0)	{ pOption[ kOption_NotAlignmentNtfSize ]	= number;}
			// tex4x4
			else if( strcmp( pChar, &optionStr[ kOption_Algorithm0 ][0]		) == 0)	{ pOption[ kOption_Algorithm0 ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_Algorithm1 ][0]		) == 0)	{ pOption[ kOption_Algorithm1 ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_Algorithm2 ][0]		) == 0)	{ pOption[ kOption_Algorithm2 ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_Algorithm3 ][0]		) == 0)	{ pOption[ kOption_Algorithm3 ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_TransparencyAlpha ][0]		) == 0)	{ pOption[ kOption_TransparencyAlpha ]	= number;}
			else if( strncmp( pChar, &optionStr[ kOption_TransparencyColor ][0], 3	) == 0)	{ pOption[ kOption_TransparencyColor ]	= number;}
			else if( strncmp( pChar, &optionStr[ kOption_CompressPalette ][0], 3	) == 0)	{ pOption[ kOption_CompressPalette ]	= number;}
			else if( strcmp( pChar, &optionStr[ kOption_NotCompressPalette ][0]		) == 0)	{ pOption[ kOption_NotCompressPalette ]	= number;}

#ifdef USE_BG
			// BG
			else if( strcmp( pChar, &optionStr[ kOption_Bg ][0]				) == 0)	{ pOption[ kOption_Bg ]				= number;}
			else if( strcmp( pChar, &optionStr[ kOption_BgFormat ][0]		) == 0)	{ pOption[ kOption_BgFormat ]		= number++;}
			else if( strcmp( pChar, &optionStr[ kOption_BgPathName ][0]		) == 0)	{ pOption[ kOption_BgPathName ]		= number++;}
			else if( strcmp( pChar, &optionStr[ kOption_BgBinary ][0]		) == 0)	{ pOption[ kOption_BgBinary ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_BgPaletteNumber ][0]) == 0)	{ pOption[ kOption_BgPaletteNumber ]= number++;}
			else if( strcmp( pChar, &optionStr[ kOption_BgNotAlignmentPaletteNumber ][0]) == 0)	{ pOption[ kOption_BgNotAlignmentPaletteNumber ]= number;}
			else if( strcmp( pChar, &optionStr[ kOption_BgNotVFlip ][0]		) == 0)	{ pOption[ kOption_BgNotVFlip ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_BgNotHFlip ][0]		) == 0)	{ pOption[ kOption_BgNotHFlip ]		= number;}
			else if( strcmp( pChar, &optionStr[ kOption_BgNotCompressChar ][0]) == 0){ pOption[ kOption_BgNotCompressChar ]	= number;}	
			else if( strcmp( pChar, &optionStr[ kOption_BgOffsetImage ][0]	) == 0){ pOption[ kOption_BgOffsetImage ]	= number++;}	
#endif
			else
				return kErrorStatus_NotOption;	// Error: option not registered
		}
		else
		{
			notCount++;
			if( notCount > 1 )		return kErrorStatus_OverInputPathNumber;		// Error: number of files input
			pOption[ kOption_InputPathName ] = number;
		}
		number++;
	}

	if( argc != number )	return kErrorStatus_NotOptionNumber;			// Error: number of options

	return kErrorStatus_Success;
}

/****************************************************************************
	afterErrorProcess()

	Process after an error
 ***************************************************************************/
static void afterErrorProcess(
		tErrorStatus			errorStatusNumber
)
{
	char errorStr[ ERRORSTATUS_STRING_MAX ];

	if( errorStatusNumber != kErrorStatus_Success )
	{
//		printHelpView();										// Display help screen
		strcpy( errorStr, pErrorStatusStr[ errorStatusNumber ]);
		fprintf( stderr, "%s", errorStr);								// Display error detail
		exit(1);	
	}
}

/****************************************************************************
	OptionProcess()

	Command line execution
 ***************************************************************************/
static tErrorStatus
OptionProcess(
		TXLibUtilityEx::tTextureData*			pTextureData,
		int*									pOption,
		int										argc,
		_TCHAR*									argv[],
		bool									textureViewerFlg

)
{
	tErrorStatus errorStatusNumber;											// Error number

    //---------------------------------------------------------------------------
    // Get texture data
    //---------------------------------------------------------------------------
	char								inputPathName[ _MAX_PATH];						// Input image file path name
	char								outputNtfPathName[ _MAX_PATH];					// Output NTF path name
	bool								ntfBinaryFlg						= false;	// Output NTF binary format flag
	bool								sizeAlignmentFlg					= true;		// Output NTF image size alignment flag
	char								outputOptionPathName[ _MAX_PATH];				// Output option file path name
	char								palettePathName[ _MAX_PATH];					// Palette file path name
	TXLib::CTexture::tType				convertTextureType	= TXLib::CTexture::kType_not;// Texture conversion format
	TXLib::CDefaultColorProcessor
		::tConvertFactor				convertFactor;						// Conversion factor

	// Clear memory
	std::memset( inputPathName,			0, _MAX_PATH);
	std::memset( outputNtfPathName,		0, _MAX_PATH);
	std::memset( outputOptionPathName,	0, _MAX_PATH);
	std::memset( palettePathName,		0, _MAX_PATH);
	std::memset( (void*)&convertFactor, 0, sizeof(TXLib::CDefaultColorProcessor::tConvertFactor));

	// Get data
	errorStatusNumber = GetInputData(	inputPathName,
										outputNtfPathName,
										&ntfBinaryFlg,
										&sizeAlignmentFlg,
										outputOptionPathName,
										palettePathName,
										&convertTextureType,
										&convertFactor,
										argv,
										pOption );

	if( errorStatusNumber !=  kErrorStatus_Success ) return errorStatusNumber;

#ifdef USE_BG
    //---------------------------------------------------------------------------
    // Get BG data
    //---------------------------------------------------------------------------
	char						outputBgPathName[_MAX_PATH];		// Output Bg Path Name
	bool						bgBinaryFlg			= false;		// Output Bg binary format flag
	TXLibUtilityEx::CBg::tType	bgType;								// BG format
	u8							bgPaletteNumber		= 0;			// Palette no. 
	bool						bgAlignmentPaletteNumberFlg	= false;	// Palette number alignment flag
	u8							offsetColorIndex	= 0;			// Color index offset value 
	bool						vFlipFlg			= false;		// VFlip flag
	bool						hFlipFlg			= false;		// HFlip flag
	bool						compressFlg			= false;		// Character compression flag 

	std::memset( outputBgPathName,		0, _MAX_PATH);

	// Get the data
	errorStatusNumber = GetInputBgData(	outputBgPathName,
										&bgBinaryFlg,
										&bgType,
										&bgPaletteNumber,
										&bgAlignmentPaletteNumberFlg,
										&offsetColorIndex,
										&vFlipFlg,
										&hFlipFlg,
										&compressFlg,
										inputPathName,
										argv,
										pOption );


	if( errorStatusNumber !=  kErrorStatus_Success ) return errorStatusNumber;
#endif

    //---------------------------------------------------------------------------
    // Use TXLib to output
    //---------------------------------------------------------------------------
	TXLib::CTexture texture;

	// Read file
	if( !TXLibUtility::Load( &texture, inputPathName ) )	return kErrorStatus_LoadFile;


	// Specify palette file
	TXLib::CTexture textureForPalette;

	if( palettePathName[0] != NULL )
	{
		if( !TXLibUtility::Load( &textureForPalette, palettePathName ) )	return kErrorStatus_LoadFile;
		convertFactor.pUsePalette = &textureForPalette.GetColorPalette();
	}

	// Set conversion factor
	texture.GetColorProcessor()->SetConvertFactor( &convertFactor);

	// Convert
	if( convertTextureType != TXLib::CTexture::kType_not )
	{
		if( !texture.Convert( convertTextureType ) )		return kErrorStatus_Convert;
	}

	// Output NTF
	if( outputNtfPathName[0] != NULL )
	{
		if( !TXLibUtility::SaveNtf( outputNtfPathName,
									&texture, 
									ntfBinaryFlg,
									inputPathName,
									textureViewerFlg,
									sizeAlignmentFlg
									))	return kErrorStatus_SaveNtfFile;
	}

	// Output option file
	if( outputOptionPathName[0] != NULL )
	{
		if(!TXLibUtility::Save( outputOptionPathName, &texture )) return kErrorStatus_SaveOptionFile;
	}

#ifdef USE_BG
	// Output BG
	if( outputBgPathName[0] != NULL )
	{
		// Specifies the default format if no BG format is specified.
		if( bgType == TXLibUtilityEx::CBg::kType_not )
		{
			switch( texture.GetType() )
			{
				case TXLib::CTexture::kType_palette16:	bgType = TXLibUtilityEx::CBg::kType_text_palette16;		break;
				case TXLib::CTexture::kType_palette256:	bgType = TXLibUtilityEx::CBg::kType_text_palette256;	break;
				case TXLib::CTexture::kType_direct:		bgType = TXLibUtilityEx::CBg::kType_exAffine_direct;	break;
				default:								return kErrorStatus_DifferentBgType; // Cannot convert
			}
		}

		u32 nChar = 0; // Text BG character count
		if(!TXLibUtilityEx::SaveBg( &nChar,
									outputBgPathName,
									bgBinaryFlg,
									&texture,
									bgType,
									hFlipFlg,
									vFlipFlg,
									compressFlg,
									bgPaletteNumber,
									bgAlignmentPaletteNumberFlg,
									offsetColorIndex,
									inputPathName
									))
		{
			printf("Number of characters:          %6d\n",nChar);
			return kErrorStatus_SaveBgFile;
		}
	}
#endif

    //---------------------------------------------------------------------------
    // Create data for TextureViewer
    //---------------------------------------------------------------------------
	if(( outputNtfPathName != NULL) && ( pTextureData != NULL ) && textureViewerFlg && !ntfBinaryFlg)
	{
		pTextureData->type		= texture.GetType();
		pTextureData->width		= texture.GetWidth();
		pTextureData->height	= texture.GetHeight();
		strcpy( pTextureData->ntfPathName, outputNtfPathName );

		TXLib::CNtf ntf;

		ntf.GetDataByteSize( 	&pTextureData->texelDataSize,
								&pTextureData->paletteDataSize,
								&pTextureData->paletteIndexDataSize,
								&texture,
								sizeAlignmentFlg
								);
	}

	return kErrorStatus_Success;
}

/****************************************************************************
	ScriptProcess()

	Script process
 ***************************************************************************/
static tErrorStatus
ScriptProcess(
	const	char*				pPathName,				// Script path name 
			char*				pExePathName,			// Executable file path name
			bool				statusViewFlg,			// Status detail display flag
			bool				textureViewerFlg		// TextureViewer data output flag 
)
{
	assert( (pPathName != NULL) && ( pExePathName != NULL ) );

	printf("Running script file %s\n\n", pPathName );

	FILE* pFin;
	tErrorStatus errorStatusNumber;							// Error number

	if( ( pFin = fopen( pPathName, "rb" ) ) == NULL ) return kErrorStatus_LoadScriptFile;

	char		lineData[ SCRIPT_LINEDATA_MAX ];	// Contents of 1 line
	std::memset( lineData, 0, sizeof( lineData ) );

	int			textureDataNumber = 0;				// Data number
	int			dataNumber=1;						// Data number 

	int			argc;								// Number of input char strings
	_TCHAR*		argv[ SCRIPT_LINEDATA_MAX ];		// Array of input char string addresses
	int			option[ OPTION_NUMBER_MAX ];		// Position of option

	// For TextureViewer
	TXLibUtilityEx::tTextureData*		pTextureDataArray = NULL;

	// Get the largest data number
	while( fgets( lineData, SCRIPT_LINEDATA_MAX, pFin ) != NULL )
	{
		textureDataNumber++;
	}

	// Allocate memory
	pTextureDataArray = new TXLibUtilityEx::tTextureData[ textureDataNumber ];

	// Restore to original value
	textureDataNumber = 0;				// Data number
	fseek( pFin, 0, SEEK_SET );

	u32 lineNumber = 0;		// Line number
	while( fgets( lineData, SCRIPT_LINEDATA_MAX, pFin ) != NULL )	// Read one line 
	{
		lineNumber++;

		// Skip comments and control characters in the script 
		if( strncmp( lineData, "//", 2) == 0) continue;
		if( strncmp( lineData, "\r\n", 2) == 0) continue;

		// Line feed code converted from "\r\n" -> NULL 
		int n = (int)strlen( lineData );
		lineData[ n-2 ] = lineData[ n-1 ] = NULL;

		// Convert from TAB -> SPACE
		char* pChar = lineData;
		while( *pChar != NULL )
		{
			if( *pChar == 0x09 )	// TAB
			{
				*pChar = ' ';
			}
			pChar++;
		}

		std::memset( argv, 0, sizeof( argv ) );		// Clear command line contents

		// Create argc, argv[]
		pChar	= strtok( lineData, " " );	// Read options delimited by SPACEs 
		argc	= 0;
		argv[ argc++ ] = pExePathName;		// Executable file name for first option

		while( pChar != NULL )
		{
			argv[ argc++ ] = pChar;
			pChar = strtok( NULL, " " );
		}

		// Get the option position from the input data 
		std::memset( option, 0, sizeof( option ) );							// Clear option
		errorStatusNumber = GetOptionInputDataNumber( option, argc, argv );	// Get option
		if( errorStatusNumber != kErrorStatus_Success ) goto error;

		// If script option is in script, error 
		if( option[ kOption_Script ] > 0 )
		{
			errorStatusNumber = kErrorStatus_DuplicateScriptFile;
			goto error;
		}

		// Display status 
		if( statusViewFlg )
		{
			printf("\n");
			printf( "Running script data number:          %4d\n", dataNumber);
			option[ kOption_View ] = 1;
		}
		else
		{
			option[ kOption_View ] = 0;
		}

		// Command line processing 
		errorStatusNumber = OptionProcess( &pTextureDataArray[textureDataNumber], option, argc, argv, textureViewerFlg );	

		if( errorStatusNumber != kErrorStatus_Success ) goto error;

		std::memset( lineData, 0, sizeof( lineData ) );		// Clear one line 

		// Count the number of NTF outputs
		if( pTextureDataArray[textureDataNumber].ntfPathName[0] != NULL )
		{
			textureDataNumber++;
		}

		// Increment the data number 
		dataNumber++;
	}

	// Create data for TextureViewer
	if( textureViewerFlg && ( textureDataNumber > 0 ) )
	{
		char driveName[_MAX_DRIVE];
		char dirName[_MAX_DIR];
		char pathName[_MAX_PATH];

		// Change the file name in the script file path.
		printf("\n");

		_splitpath( pPathName, driveName, dirName, NULL, NULL );
		strcpy( pathName, driveName );
		strcat( pathName, dirName );
		strcat( pathName, "textureData.h");
		TXLibUtilityEx::SaveNtfHeaderFile( pathName, pTextureDataArray, textureDataNumber );				// Create header file
		printf( "Texture data header file name: %s\n", pathName );

		TXLibUtility::ChangeExtName( pathName, ".c" );
		TXLibUtilityEx::SaveNtfImageParameterFile( pathName, pTextureDataArray, textureDataNumber);		// Create texture parameter file
		printf( "Texture parameter file name:   %s\n", pathName );
	}

	errorStatusNumber = kErrorStatus_Success;	// Success
error:
	if( errorStatusNumber != kErrorStatus_Success )
	{	// Display line number where error occurred 
		printf("\nError while running script file\n");
		printf("Running script data number        %4d:\n", dataNumber);
		printf("Running script line number        %4d:\n\n", lineNumber );
	}

	delete[] pTextureDataArray;
	fclose(pFin);
	return errorStatusNumber;
}

/****************************************************************************
	_tmain()

	Main
 ***************************************************************************/
int _tmain(int argc, _TCHAR* argv[])
{
	tErrorStatus errorStatusNumber;							// Error number
	int	option[ OPTION_NUMBER_MAX ];						// Position of option
	std::memset( option, 0, sizeof( option ) );

	// Display ntexconv title
	puts("ntexconv.exe Version 1.5.0\n\n");

#ifdef TLB_NTEXCONV_PRINT_TIME
	// For calculation of process time
	clock_t start_time, end_time;
   start_time = clock();
#endif

	// Get the option input number
	errorStatusNumber = GetOptionInputDataNumber( option, argc, argv );	
	afterErrorProcess( errorStatusNumber );						// Processing after error

	u32		optionNumber;		// Position of option

	// Help screen
	if( option[ kOption_Help ] > 0 )
	{
		printHelpView();				// Display help screen 
		exit(1);
	}

	bool textureViewerFlg	= ( option[ kOption_TextureViewer ] > 0 ) ? true: false;

	if( ( optionNumber = option[ kOption_Script ]) > 0 )	// Script? 
	{
		// Command line processing from script file  (Multiple command line processing)
		bool statusViewFlg		= ( option[ kOption_View ] > 0 )? true: false;

		errorStatusNumber = ScriptProcess( argv[ optionNumber + 1 ], argv[0], statusViewFlg, textureViewerFlg );
		afterErrorProcess( errorStatusNumber );							// Processing after error
	}
	else
	{
		// Command line processing without script  (Single command line processing)
		TXLibUtilityEx::tTextureData	textureData;

		errorStatusNumber = OptionProcess( &textureData, option, argc, argv, textureViewerFlg );
		afterErrorProcess( errorStatusNumber );							// Processing after error

		if( textureViewerFlg && ( textureData.ntfPathName[0] != NULL ) )
		{
			char driveName[_MAX_DRIVE];
			char dirName[_MAX_DIR];
			char pathName[_MAX_PATH];

			// Change the file name in the script file path.
			printf("\n");

			_splitpath( textureData.ntfPathName, driveName, dirName, NULL, NULL );
			strcpy( pathName, driveName );
			strcat( pathName, dirName );
			strcat( pathName, "textureData.h");
			TXLibUtilityEx::SaveNtfHeaderFile( pathName, &textureData, 1 );				// Create header file
			printf( "Texture data header file name: %s\n", pathName );

			TXLibUtility::ChangeExtName( pathName, ".c" );
			TXLibUtilityEx::SaveNtfImageParameterFile( pathName, &textureData, 1);		// Create texture parameter file
			printf( "Texture parameter file name:   %s\n", pathName );
		}
	}

#ifdef TLB_NTEXCONV_PRINT_TIME
	// For calculation of process time
	double	timeCount;
	end_time = clock();
	timeCount = (double)(end_time - start_time) / CLOCKS_PER_SEC;
	printf( "process time %2.1f\n", timeCount );
#endif

	return 0;
}

